# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The tsplink commands module.

These commands are used in the following models:
DAQ6510

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - tsplink.group
    - tsplink.initialize()
    - tsplink.line[N].reset()
    - tsplink.line[N].state
    - tsplink.master
    - tsplink.node
    - tsplink.readport()
    - tsplink.state
    - tsplink.writeport()
"""
from typing import Dict, Optional, TYPE_CHECKING, Union

from .._helpers import (
    BaseTSPCmd,
    DefaultDictPassKeyToFactory,
    NoDeviceProvidedError,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class TsplinkLineItem(ValidatedDynamicNumberCmd, BaseTSPCmd):
    """The ``tsplink.line[N]`` command tree.

    **Info:**
        - ``N``, the trigger line (1 to 3).

    Properties/methods:
        - ``.reset()``: The ``tsplink.line[N].reset()`` function.
        - ``.state``: The ``tsplink.line[N].state`` attribute.
    """

    @property
    def state(self) -> str:
        """Access the ``tsplink.line[N].state`` attribute.

        **Description:**
            - This attribute reads or writes the digital state of a TSP-Link synchronization line.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.line[N].state)`` query.
            - Setting this property to a value will send the ``tsplink.line[N].state = value``
              command.

        **TSP Syntax:**

        ::

            - tsplink.line[N].state = value
            - print(tsplink.line[N].state)

        **Info:**
            - ``N``, the trigger line (1 to 3).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".state"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.state)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.state`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @state.setter
    def state(self, value: Union[str, float]) -> None:
        """Access the ``tsplink.line[N].state`` attribute.

        **Description:**
            - This attribute reads or writes the digital state of a TSP-Link synchronization line.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.line[N].state)`` query.
            - Setting this property to a value will send the ``tsplink.line[N].state = value``
              command.

        **TSP Syntax:**

        ::

            - tsplink.line[N].state = value
            - print(tsplink.line[N].state)

        **Info:**
            - ``N``, the trigger line (1 to 3).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".state", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.state = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.state`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def reset(self) -> None:
        """Run the ``tsplink.line[N].reset()`` function.

        **Description:**
            - This function resets some of the TSP-Link trigger attributes to their factory
              defaults.

        **TSP Syntax:**

        ::

            - tsplink.line[N].reset()

        **Info:**
            - ``N``, the trigger line (1 to 3).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.reset()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.reset()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Tsplink(BaseTSPCmd):
    """The ``tsplink`` command tree.

    Properties/methods:
        - ``.group``: The ``tsplink.group`` attribute.
        - ``.initialize()``: The ``tsplink.initialize()`` function.
        - ``.line``: The ``tsplink.line[N]`` command tree.
        - ``.master``: The ``tsplink.master`` attribute.
        - ``.node``: The ``tsplink.node`` attribute.
        - ``.readport()``: The ``tsplink.readport()`` function.
        - ``.state``: The ``tsplink.state`` attribute.
        - ``.writeport()``: The ``tsplink.writeport()`` function.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "tsplink") -> None:
        super().__init__(device, cmd_syntax)
        self._line: Dict[int, TsplinkLineItem] = DefaultDictPassKeyToFactory(
            lambda x: TsplinkLineItem(device, f"{self._cmd_syntax}.line[{x}]")
        )

    @property
    def group(self) -> str:
        """Access the ``tsplink.group`` attribute.

        **Description:**
            - This attribute contains the group number of a TSP-Link node.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.group)`` query.
            - Setting this property to a value will send the ``tsplink.group = value`` command.

        **TSP Syntax:**

        ::

            - tsplink.group = value
            - print(tsplink.group)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".group"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.group)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.group`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @group.setter
    def group(self, value: Union[str, float]) -> None:
        """Access the ``tsplink.group`` attribute.

        **Description:**
            - This attribute contains the group number of a TSP-Link node.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.group)`` query.
            - Setting this property to a value will send the ``tsplink.group = value`` command.

        **TSP Syntax:**

        ::

            - tsplink.group = value
            - print(tsplink.group)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".group", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.group = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.group`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def line(self) -> Dict[int, TsplinkLineItem]:
        """Return the ``tsplink.line[N]`` command tree.

        **Info:**
            - ``N``, the trigger line (1 to 3).

        Sub-properties/methods:
            - ``.reset()``: The ``tsplink.line[N].reset()`` function.
            - ``.state``: The ``tsplink.line[N].state`` attribute.
        """
        return self._line

    @property
    def master(self) -> str:
        """Access the ``tsplink.master`` attribute.

        **Description:**
            - This attribute reads the node number assigned to the master node.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.master)`` query.

        **TSP Syntax:**

        ::

            - print(tsplink.master)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".master"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.master)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.master`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def node(self) -> str:
        """Access the ``tsplink.node`` attribute.

        **Description:**
            - This attribute defines the node number.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.node)`` query.
            - Setting this property to a value will send the ``tsplink.node = value`` command.

        **TSP Syntax:**

        ::

            - tsplink.node = value
            - print(tsplink.node)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".node"
            return self._device.query(f"print({self._cmd_syntax}.node)")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.node`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @node.setter
    def node(self, value: Union[str, float]) -> None:
        """Access the ``tsplink.node`` attribute.

        **Description:**
            - This attribute defines the node number.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.node)`` query.
            - Setting this property to a value will send the ``tsplink.node = value`` command.

        **TSP Syntax:**

        ::

            - tsplink.node = value
            - print(tsplink.node)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".node", value
                )
            else:
                self._device.write(f"{self._cmd_syntax}.node = {value}")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.node`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def state(self) -> str:
        """Access the ``tsplink.state`` attribute.

        **Description:**
            - This attribute describes the TSP-Link online state.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.state)`` query.

        **TSP Syntax:**

        ::

            - print(tsplink.state)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".state"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.state)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.state`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def initialize(self, expected_nodes: Optional[int] = None) -> str:
        """Run the ``tsplink.initialize()`` function.

        **Description:**
            - This function initializes all instruments and enclosures in the TSP-Link system.

        **TSP Syntax:**

        ::

            - tsplink.initialize()

        Args:
            expected_nodes (optional): The number of nodes expected on the system (1 to 32).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(str(x) for x in (expected_nodes,) if x is not None)
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.initialize({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.initialize()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def readport(self) -> str:
        """Run the ``tsplink.readport()`` function.

        **Description:**
            - This function reads the TSP-Link trigger lines as a digital I/O port.

        **TSP Syntax:**

        ::

            - tsplink.readport()

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.readport())"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.readport()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def writeport(self, data: int) -> None:
        """Run the ``tsplink.writeport()`` function.

        **Description:**
            - This function writes to all TSP-Link synchronization lines as a digital I/O port.

        **TSP Syntax:**

        ::

            - tsplink.writeport()

        Args:
            data: Value to write to the port (0 to 7).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.writeport({data})")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.writeport()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""SMU2400Interactive device driver module."""

from abc import ABC
from functools import cached_property
from typing import Tuple, Union

from tm_devices.commands import (
    SMU2450Commands,
    SMU2460Commands,
    SMU2461Commands,
    SMU2470Commands,
)
from tm_devices.drivers.device import family_base_class
from tm_devices.drivers.pi._ieee488_2_commands import LegacyTSPIEEE4882Commands
from tm_devices.drivers.pi.source_measure_units.source_measure_unit import SourceMeasureUnit


@family_base_class
class SMU2400Interactive(SourceMeasureUnit, ABC):
    """Base SMU2400Interactive device driver."""

    _IEEE_COMMANDS_CLASS = LegacyTSPIEEE4882Commands

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Properties
    ################################################################################################
    @property
    def all_channel_names_list(self) -> Tuple[str, ...]:
        """Return a tuple containing all the channel names."""
        return tuple(f"OUTPUT{x+1}" for x in range(self.total_channels))

    @property
    def commands(
        self,
    ) -> Union[SMU2450Commands, SMU2460Commands, SMU2461Commands, SMU2470Commands]:
        """Return the device commands."""
        return self._commands  # pragma: no cover

    @property
    def ieee_cmds(self) -> LegacyTSPIEEE4882Commands:
        """Return an internal class containing methods for the standard IEEE 488.2 command set."""
        return self._ieee_cmds  # type: ignore

    @cached_property
    def total_channels(self) -> int:
        """Return the total number of channels (all types)."""
        return 1

    ################################################################################################
    # Public Methods
    ################################################################################################
    def get_eventlog_status(self) -> Tuple[bool, str]:
        """Help function for getting the eventlog status.

        Returns:
            Boolean indicating no error, String containing concatenated contents of event log.
        """
        result_allev = False
        allev_result_str = '0,"No events to report - queue empty"'

        if not (err_count := int(self.query("print(eventlog.getcount(eventlog.SEV_ERROR))"))):
            result_allev = True
        else:
            allev_result_list = [self.query("print(eventlog.next())") for _ in range(err_count)]
            allev_result_str = ",".join(allev_result_list)
        return result_allev, allev_result_str

    ################################################################################################
    # Private Methods
    ################################################################################################
    def _reboot(self) -> None:
        """Perform the actual rebooting code."""
        # TODO: implement
        raise NotImplementedError(
            f"``.reboot()`` is not yet implemented for the {self.__class__.__name__} driver"
        )



# pylint: disable=too-many-lines
"""The mmemory commands module.

These commands are used in the following models:
AWG5K, AWG5KC, AWG7K, AWG7KC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - MMEMory:CATalog? [<msus>]
    - MMEMory:CDIRectory [<directory_name>]
    - MMEMory:CDIRectory?
    - MMEMory:DATA <file_name>,<block_data>? <file_name>
    - MMEMory:DELete <file_name>[,<msus>]
    - MMEMory:EXPort <wfm_name>,<filename>,<type>
    - MMEMory:IMPort <wfm_name>,<filename>,<type>
    - MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe <state>
    - MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe?
    - MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel <NR1>
    - MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel?
    - MMEMory:IMPort:PARameter:LEVel:UPDate:STATe <state>
    - MMEMory:IMPort:PARameter:LEVel:UPDate:STATe?
    - MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE <Type>
    - MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE?
    - MMEMory:IMPort:PARameter:NORMalize {NONE|FSCale|ZREFerence}
    - MMEMory:IMPort:PARameter:NORMalize?
    - MMEMory:IMPort:PARameter:RESampling:FREQuency <NR3>
    - MMEMory:IMPort:PARameter:RESampling:FREQuency?
    - MMEMory:IMPort:PARameter:RESampling:STATe <state>
    - MMEMory:IMPort:PARameter:RESampling:STATe?
    - MMEMory:MDIRectory <directory_name>
    - MMEMory:MSIS [<msus>]
    - MMEMory:MSIS?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdReadWithArguments, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class MmemoryMsis(SCPICmdWrite, SCPICmdRead):
    """The ``MMEMory:MSIS`` command.

    **Description:**
        - This command selects or returns a mass storage device used by all MMEMory commands. <msus>
          specifies a drive using a drive letter. The drive letter can represent hard disk drives,
          network drives, external DVD/CD-RW drives, or USB memory.

    **Usage:**
        - Using the ``.query()`` method will send the ``MMEMory:MSIS?`` query.
        - Using the ``.verify(value)`` method will send the ``MMEMory:MSIS?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MMEMory:MSIS value`` command.

    **SCPI Syntax:**

    ::

        - MMEMory:MSIS [<msus>]
        - MMEMory:MSIS?
    """


class MmemoryMdirectory(SCPICmdWrite):
    """The ``MMEMory:MDIRectory`` command.

    **Description:**
        - This command creates a directory in the mass storage system. If the specified directory is
          locked in the mass storage system, this command causes an error.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``MMEMory:MDIRectory value`` command.

    **SCPI Syntax:**

    ::

        - MMEMory:MDIRectory <directory_name>
    """


class MmemoryImportParameterResamplingState(SCPICmdWrite, SCPICmdRead):
    """The ``MMEMory:IMPort:PARameter:RESampling:STATe`` command.

    **Description:**
        - This command sets or queries the resampling state for waveform import. This command is
          effective only when the following conditions are met: Waveform data to be imported must
          have sampling rate information. ``MMEMORY:IMPORT:PARAMETER:FREQUENCY:UPDATE:STATE``
          command must be set to True. Use this command to set the resampling state on or off. If
          you set the resampling state on, resampling is automatically invoked when importing
          waveform data. The query form of this command returns the resampling state of the
          instrument.

    **Usage:**
        - Using the ``.query()`` method will send the ``MMEMory:IMPort:PARameter:RESampling:STATe?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``MMEMory:IMPort:PARameter:RESampling:STATe?`` query and raise an AssertionError if the
          returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``MMEMory:IMPort:PARameter:RESampling:STATe value`` command.

    **SCPI Syntax:**

    ::

        - MMEMory:IMPort:PARameter:RESampling:STATe <state>
        - MMEMory:IMPort:PARameter:RESampling:STATe?

    **Info:**
        - ``<state>`` ::=<Boolean>.
    """


class MmemoryImportParameterResamplingFrequency(SCPICmdWrite, SCPICmdRead):
    """The ``MMEMory:IMPort:PARameter:RESampling:FREQuency`` command.

    **Description:**
        - This command sets or queries the sampling rate parameter for resampling. The specified
          sampling rate is applied to imported waveform.

    **Usage:**
        - Using the ``.query()`` method will send the
          ``MMEMory:IMPort:PARameter:RESampling:FREQuency?`` query.
        - Using the ``.verify(value)`` method will send the
          ``MMEMory:IMPort:PARameter:RESampling:FREQuency?`` query and raise an AssertionError if
          the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``MMEMory:IMPort:PARameter:RESampling:FREQuency value`` command.

    **SCPI Syntax:**

    ::

        - MMEMory:IMPort:PARameter:RESampling:FREQuency <NR3>
        - MMEMory:IMPort:PARameter:RESampling:FREQuency?

    **Info:**
        - ``<NR3>``
    """


class MmemoryImportParameterResampling(SCPICmdRead):
    """The ``MMEMory:IMPort:PARameter:RESampling`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``MMEMory:IMPort:PARameter:RESampling?``
          query.
        - Using the ``.verify(value)`` method will send the ``MMEMory:IMPort:PARameter:RESampling?``
          query and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.frequency``: The ``MMEMory:IMPort:PARameter:RESampling:FREQuency`` command.
        - ``.state``: The ``MMEMory:IMPort:PARameter:RESampling:STATe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._frequency = MmemoryImportParameterResamplingFrequency(
            device, f"{self._cmd_syntax}:FREQuency"
        )
        self._state = MmemoryImportParameterResamplingState(device, f"{self._cmd_syntax}:STATe")

    @property
    def frequency(self) -> MmemoryImportParameterResamplingFrequency:
        """Return the ``MMEMory:IMPort:PARameter:RESampling:FREQuency`` command.

        **Description:**
            - This command sets or queries the sampling rate parameter for resampling. The specified
              sampling rate is applied to imported waveform.

        **Usage:**
            - Using the ``.query()`` method will send the
              ``MMEMory:IMPort:PARameter:RESampling:FREQuency?`` query.
            - Using the ``.verify(value)`` method will send the
              ``MMEMory:IMPort:PARameter:RESampling:FREQuency?`` query and raise an AssertionError
              if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``MMEMory:IMPort:PARameter:RESampling:FREQuency value`` command.

        **SCPI Syntax:**

        ::

            - MMEMory:IMPort:PARameter:RESampling:FREQuency <NR3>
            - MMEMory:IMPort:PARameter:RESampling:FREQuency?

        **Info:**
            - ``<NR3>``
        """
        return self._frequency

    @property
    def state(self) -> MmemoryImportParameterResamplingState:
        """Return the ``MMEMory:IMPort:PARameter:RESampling:STATe`` command.

        **Description:**
            - This command sets or queries the resampling state for waveform import. This command is
              effective only when the following conditions are met: Waveform data to be imported
              must have sampling rate information.
              ``MMEMORY:IMPORT:PARAMETER:FREQUENCY:UPDATE:STATE`` command must be set to True. Use
              this command to set the resampling state on or off. If you set the resampling state
              on, resampling is automatically invoked when importing waveform data. The query form
              of this command returns the resampling state of the instrument.

        **Usage:**
            - Using the ``.query()`` method will send the
              ``MMEMory:IMPort:PARameter:RESampling:STATe?`` query.
            - Using the ``.verify(value)`` method will send the
              ``MMEMory:IMPort:PARameter:RESampling:STATe?`` query and raise an AssertionError if
              the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``MMEMory:IMPort:PARameter:RESampling:STATe value`` command.

        **SCPI Syntax:**

        ::

            - MMEMory:IMPort:PARameter:RESampling:STATe <state>
            - MMEMory:IMPort:PARameter:RESampling:STATe?

        **Info:**
            - ``<state>`` ::=<Boolean>.
        """
        return self._state


class MmemoryImportParameterNormalize(SCPICmdWrite, SCPICmdRead):
    """The ``MMEMory:IMPort:PARameter:NORMalize`` command.

    **Description:**
        - This command sets or queries if the imported data is normalized during text data import
          operation. The imported waveform data is normalized based on the option set in this
          command. When ZREFerence is selected, the offset is preserved during normalization
          operation. If FSCale is selected, offset is lost and full scale of the DAC is used for
          normalization. NONE will not normalize the data.

    **Usage:**
        - Using the ``.query()`` method will send the ``MMEMory:IMPort:PARameter:NORMalize?`` query.
        - Using the ``.verify(value)`` method will send the ``MMEMory:IMPort:PARameter:NORMalize?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``MMEMory:IMPort:PARameter:NORMalize value`` command.

    **SCPI Syntax:**

    ::

        - MMEMory:IMPort:PARameter:NORMalize {NONE|FSCale|ZREFerence}
        - MMEMory:IMPort:PARameter:NORMalize?

    **Info:**
        - ``<Normalization_type>``
        - ``NONE`` indicates that the imported data is not normalized.
        - ``FSCale`` indicates that the imported data is normalized with full DAC range.
        - ``ZREFerence`` indicates that the imported data is normalized with offset preserved.
    """


class MmemoryImportParameterLevelUpdateType(SCPICmdWrite, SCPICmdRead):
    r"""The ``MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE`` command.

    **Description:**
        - This commands sets or queries the data to be imported. It also sets or queries which
          data's amplitude and offset values are selected for update during RSA file import. This
          command is effective only when ``MMEMORY:IMPORT:PARAMETER:LEVEL:UPDATE:STATE`` is set to
          True and IQT or TIQ is selected as the <Type> for the ``MMEMORY:IMPORT`` command.

    **Usage:**
        - Using the ``.query()`` method will send the
          ``MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE?`` query.
        - Using the ``.verify(value)`` method will send the
          ``MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE?`` query and raise an AssertionError if the
          returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE value`` command.

    **SCPI Syntax:**

    ::

        - MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE <Type>
        - MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE?

    **Info:**
        - ``<Type>`` ::={IDATa\|QDATa}.
        - ``IDATa`` indicates that the instrument imports I data.
        - ``QDATa`` indicates that the instrument imports Q data.
    """


class MmemoryImportParameterLevelUpdateState(SCPICmdWrite, SCPICmdRead):
    """The ``MMEMory:IMPort:PARameter:LEVel:UPDate:STATe`` command.

    **Description:**
        - This command sets or queries the LEVel parameter which determines whether amplitude and
          offsets are modified during waveform import. If this value is set, the instrument
          amplitude and offset are automatically updated during waveform import.

    **Usage:**
        - Using the ``.query()`` method will send the
          ``MMEMory:IMPort:PARameter:LEVel:UPDate:STATe?`` query.
        - Using the ``.verify(value)`` method will send the
          ``MMEMory:IMPort:PARameter:LEVel:UPDate:STATe?`` query and raise an AssertionError if the
          returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``MMEMory:IMPort:PARameter:LEVel:UPDate:STATe value`` command.

    **SCPI Syntax:**

    ::

        - MMEMory:IMPort:PARameter:LEVel:UPDate:STATe <state>
        - MMEMory:IMPort:PARameter:LEVel:UPDate:STATe?

    **Info:**
        - ``<state>`` ::=<Boolean>.
        - ``0`` indicates OFF.
        - ``1`` indicates ON.
    """


class MmemoryImportParameterLevelUpdateChannel(SCPICmdWrite, SCPICmdRead):
    """The ``MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel`` command.

    **Description:**
        - This command sets or queries the channel for which the amplitude and offset values will be
          updated during import.

    **Usage:**
        - Using the ``.query()`` method will send the
          ``MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel?`` query.
        - Using the ``.verify(value)`` method will send the
          ``MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel?`` query and raise an AssertionError if
          the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel value`` command.

    **SCPI Syntax:**

    ::

        - MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel <NR1>
        - MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel?

    **Info:**
        - ``<NR1>``
    """


class MmemoryImportParameterLevelUpdate(SCPICmdRead):
    """The ``MMEMory:IMPort:PARameter:LEVel:UPDate`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``MMEMory:IMPort:PARameter:LEVel:UPDate?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``MMEMory:IMPort:PARameter:LEVel:UPDate?`` query and raise an AssertionError if the
          returned value does not match ``value``.

    Properties:
        - ``.channel``: The ``MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel`` command.
        - ``.type``: The ``MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE`` command.
        - ``.state``: The ``MMEMory:IMPort:PARameter:LEVel:UPDate:STATe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._channel = MmemoryImportParameterLevelUpdateChannel(
            device, f"{self._cmd_syntax}:CHANnel"
        )
        self._type = MmemoryImportParameterLevelUpdateType(device, f"{self._cmd_syntax}:TYPE")
        self._state = MmemoryImportParameterLevelUpdateState(device, f"{self._cmd_syntax}:STATe")

    @property
    def channel(self) -> MmemoryImportParameterLevelUpdateChannel:
        """Return the ``MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel`` command.

        **Description:**
            - This command sets or queries the channel for which the amplitude and offset values
              will be updated during import.

        **Usage:**
            - Using the ``.query()`` method will send the
              ``MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel?`` query.
            - Using the ``.verify(value)`` method will send the
              ``MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel?`` query and raise an AssertionError
              if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel value`` command.

        **SCPI Syntax:**

        ::

            - MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel <NR1>
            - MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel?

        **Info:**
            - ``<NR1>``
        """
        return self._channel

    @property
    def type(self) -> MmemoryImportParameterLevelUpdateType:
        r"""Return the ``MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE`` command.

        **Description:**
            - This commands sets or queries the data to be imported. It also sets or queries which
              data's amplitude and offset values are selected for update during RSA file import.
              This command is effective only when ``MMEMORY:IMPORT:PARAMETER:LEVEL:UPDATE:STATE`` is
              set to True and IQT or TIQ is selected as the <Type> for the ``MMEMORY:IMPORT``
              command.

        **Usage:**
            - Using the ``.query()`` method will send the
              ``MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE?`` query.
            - Using the ``.verify(value)`` method will send the
              ``MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE?`` query and raise an AssertionError if
              the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE value`` command.

        **SCPI Syntax:**

        ::

            - MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE <Type>
            - MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE?

        **Info:**
            - ``<Type>`` ::={IDATa\|QDATa}.
            - ``IDATa`` indicates that the instrument imports I data.
            - ``QDATa`` indicates that the instrument imports Q data.
        """
        return self._type

    @property
    def state(self) -> MmemoryImportParameterLevelUpdateState:
        """Return the ``MMEMory:IMPort:PARameter:LEVel:UPDate:STATe`` command.

        **Description:**
            - This command sets or queries the LEVel parameter which determines whether amplitude
              and offsets are modified during waveform import. If this value is set, the instrument
              amplitude and offset are automatically updated during waveform import.

        **Usage:**
            - Using the ``.query()`` method will send the
              ``MMEMory:IMPort:PARameter:LEVel:UPDate:STATe?`` query.
            - Using the ``.verify(value)`` method will send the
              ``MMEMory:IMPort:PARameter:LEVel:UPDate:STATe?`` query and raise an AssertionError if
              the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``MMEMory:IMPort:PARameter:LEVel:UPDate:STATe value`` command.

        **SCPI Syntax:**

        ::

            - MMEMory:IMPort:PARameter:LEVel:UPDate:STATe <state>
            - MMEMory:IMPort:PARameter:LEVel:UPDate:STATe?

        **Info:**
            - ``<state>`` ::=<Boolean>.
            - ``0`` indicates OFF.
            - ``1`` indicates ON.
        """
        return self._state


class MmemoryImportParameterLevel(SCPICmdRead):
    """The ``MMEMory:IMPort:PARameter:LEVel`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``MMEMory:IMPort:PARameter:LEVel?`` query.
        - Using the ``.verify(value)`` method will send the ``MMEMory:IMPort:PARameter:LEVel?``
          query and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.update``: The ``MMEMory:IMPort:PARameter:LEVel:UPDate`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._update = MmemoryImportParameterLevelUpdate(device, f"{self._cmd_syntax}:UPDate")

    @property
    def update(self) -> MmemoryImportParameterLevelUpdate:
        """Return the ``MMEMory:IMPort:PARameter:LEVel:UPDate`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``MMEMory:IMPort:PARameter:LEVel:UPDate?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``MMEMory:IMPort:PARameter:LEVel:UPDate?`` query and raise an AssertionError if the
              returned value does not match ``value``.

        Sub-properties:
            - ``.channel``: The ``MMEMory:IMPort:PARameter:LEVel:UPDate:CHANnel`` command.
            - ``.type``: The ``MMEMory:IMPort:PARameter:LEVel:UPDate:TYPE`` command.
            - ``.state``: The ``MMEMory:IMPort:PARameter:LEVel:UPDate:STATe`` command.
        """
        return self._update


class MmemoryImportParameterFrequencyUpdateState(SCPICmdWrite, SCPICmdRead):
    """The ``MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe`` command.

    **Description:**
        - This command sets or queries the FREQuency parameter which determines whether frequency is
          modified during waveform import. If this value is set, the sampling rate is automatically
          updated during waveform import.

    **Usage:**
        - Using the ``.query()`` method will send the
          ``MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe?`` query.
        - Using the ``.verify(value)`` method will send the
          ``MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe?`` query and raise an AssertionError if
          the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe value`` command.

    **SCPI Syntax:**

    ::

        - MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe <state>
        - MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe?

    **Info:**
        - ``<state>`` ::=<Boolean>.
        - ``0`` indicates OFF.
        - ``1`` indicates ON.
    """


class MmemoryImportParameterFrequencyUpdate(SCPICmdRead):
    """The ``MMEMory:IMPort:PARameter:FREQuency:UPDate`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``MMEMory:IMPort:PARameter:FREQuency:UPDate?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``MMEMory:IMPort:PARameter:FREQuency:UPDate?`` query and raise an AssertionError if the
          returned value does not match ``value``.

    Properties:
        - ``.state``: The ``MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._state = MmemoryImportParameterFrequencyUpdateState(
            device, f"{self._cmd_syntax}:STATe"
        )

    @property
    def state(self) -> MmemoryImportParameterFrequencyUpdateState:
        """Return the ``MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe`` command.

        **Description:**
            - This command sets or queries the FREQuency parameter which determines whether
              frequency is modified during waveform import. If this value is set, the sampling rate
              is automatically updated during waveform import.

        **Usage:**
            - Using the ``.query()`` method will send the
              ``MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe?`` query.
            - Using the ``.verify(value)`` method will send the
              ``MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe?`` query and raise an AssertionError
              if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe value`` command.

        **SCPI Syntax:**

        ::

            - MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe <state>
            - MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe?

        **Info:**
            - ``<state>`` ::=<Boolean>.
            - ``0`` indicates OFF.
            - ``1`` indicates ON.
        """
        return self._state


class MmemoryImportParameterFrequency(SCPICmdRead):
    """The ``MMEMory:IMPort:PARameter:FREQuency`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``MMEMory:IMPort:PARameter:FREQuency?`` query.
        - Using the ``.verify(value)`` method will send the ``MMEMory:IMPort:PARameter:FREQuency?``
          query and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.update``: The ``MMEMory:IMPort:PARameter:FREQuency:UPDate`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._update = MmemoryImportParameterFrequencyUpdate(device, f"{self._cmd_syntax}:UPDate")

    @property
    def update(self) -> MmemoryImportParameterFrequencyUpdate:
        """Return the ``MMEMory:IMPort:PARameter:FREQuency:UPDate`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the
              ``MMEMory:IMPort:PARameter:FREQuency:UPDate?`` query.
            - Using the ``.verify(value)`` method will send the
              ``MMEMory:IMPort:PARameter:FREQuency:UPDate?`` query and raise an AssertionError if
              the returned value does not match ``value``.

        Sub-properties:
            - ``.state``: The ``MMEMory:IMPort:PARameter:FREQuency:UPDate:STATe`` command.
        """
        return self._update


class MmemoryImportParameter(SCPICmdRead):
    """The ``MMEMory:IMPort:PARameter`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``MMEMory:IMPort:PARameter?`` query.
        - Using the ``.verify(value)`` method will send the ``MMEMory:IMPort:PARameter?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.frequency``: The ``MMEMory:IMPort:PARameter:FREQuency`` command tree.
        - ``.level``: The ``MMEMory:IMPort:PARameter:LEVel`` command tree.
        - ``.normalize``: The ``MMEMory:IMPort:PARameter:NORMalize`` command.
        - ``.resampling``: The ``MMEMory:IMPort:PARameter:RESampling`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._frequency = MmemoryImportParameterFrequency(device, f"{self._cmd_syntax}:FREQuency")
        self._level = MmemoryImportParameterLevel(device, f"{self._cmd_syntax}:LEVel")
        self._normalize = MmemoryImportParameterNormalize(device, f"{self._cmd_syntax}:NORMalize")
        self._resampling = MmemoryImportParameterResampling(
            device, f"{self._cmd_syntax}:RESampling"
        )

    @property
    def frequency(self) -> MmemoryImportParameterFrequency:
        """Return the ``MMEMory:IMPort:PARameter:FREQuency`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``MMEMory:IMPort:PARameter:FREQuency?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``MMEMory:IMPort:PARameter:FREQuency?`` query and raise an AssertionError if the
              returned value does not match ``value``.

        Sub-properties:
            - ``.update``: The ``MMEMory:IMPort:PARameter:FREQuency:UPDate`` command tree.
        """
        return self._frequency

    @property
    def level(self) -> MmemoryImportParameterLevel:
        """Return the ``MMEMory:IMPort:PARameter:LEVel`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``MMEMory:IMPort:PARameter:LEVel?`` query.
            - Using the ``.verify(value)`` method will send the ``MMEMory:IMPort:PARameter:LEVel?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.update``: The ``MMEMory:IMPort:PARameter:LEVel:UPDate`` command tree.
        """
        return self._level

    @property
    def normalize(self) -> MmemoryImportParameterNormalize:
        """Return the ``MMEMory:IMPort:PARameter:NORMalize`` command.

        **Description:**
            - This command sets or queries if the imported data is normalized during text data
              import operation. The imported waveform data is normalized based on the option set in
              this command. When ZREFerence is selected, the offset is preserved during
              normalization operation. If FSCale is selected, offset is lost and full scale of the
              DAC is used for normalization. NONE will not normalize the data.

        **Usage:**
            - Using the ``.query()`` method will send the ``MMEMory:IMPort:PARameter:NORMalize?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``MMEMory:IMPort:PARameter:NORMalize?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``MMEMory:IMPort:PARameter:NORMalize value`` command.

        **SCPI Syntax:**

        ::

            - MMEMory:IMPort:PARameter:NORMalize {NONE|FSCale|ZREFerence}
            - MMEMory:IMPort:PARameter:NORMalize?

        **Info:**
            - ``<Normalization_type>``
            - ``NONE`` indicates that the imported data is not normalized.
            - ``FSCale`` indicates that the imported data is normalized with full DAC range.
            - ``ZREFerence`` indicates that the imported data is normalized with offset preserved.
        """
        return self._normalize

    @property
    def resampling(self) -> MmemoryImportParameterResampling:
        """Return the ``MMEMory:IMPort:PARameter:RESampling`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``MMEMory:IMPort:PARameter:RESampling?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``MMEMory:IMPort:PARameter:RESampling?`` query and raise an AssertionError if the
              returned value does not match ``value``.

        Sub-properties:
            - ``.frequency``: The ``MMEMory:IMPort:PARameter:RESampling:FREQuency`` command.
            - ``.state``: The ``MMEMory:IMPort:PARameter:RESampling:STATe`` command.
        """
        return self._resampling


class MmemoryImport(SCPICmdWrite, SCPICmdRead):
    r"""The ``MMEMory:IMPort`` command.

    **Description:**
        - This command imports a file into the arbitrary waveform generator's setup as a waveform.
          The supported file formats are: ISF - TDS3000 and DPO4000 waveform format TDS -
          TDS5000/TDS6000/TDS7000, DPO7000/DPO70000/DSA70000 Series waveform TXT - Text file with
          analog data TXT8 - Text file with 8-bit DAC resolution TXT10 - Text file with 10-bit DAC
          resolution TXT14 - Text file with 14-bit DAC resolution WFM - AWG400/AWG500/AWG600/AWG700
          Series waveform PAT - AWG400/AWG500/AWG600/AWG700 Series pattern file TFW - AFG3000 Series
          waveform file format IQT - RSA3000 Series waveform file format TIQ - RSA6000 Series
          waveform file format

    **Usage:**
        - Using the ``.write(value)`` method will send the ``MMEMory:IMPort value`` command.

    **SCPI Syntax:**

    ::

        - MMEMory:IMPort <wfm_name>,<filename>,<type>

    **Info:**
        - ``<wfm_name>,<filename>,<type>``
        - ``<wfm_name>`` ::=<string>.
        - ``<filename>`` ::=<string>.
        - ``<type>`` = {ISF\|TDS\|TXT\|TXT8\|TXT10\|TXT14\|WFM\|PAT\|TFW}.

    Properties:
        - ``.parameter``: The ``MMEMory:IMPort:PARameter`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._parameter = MmemoryImportParameter(device, f"{self._cmd_syntax}:PARameter")

    @property
    def parameter(self) -> MmemoryImportParameter:
        """Return the ``MMEMory:IMPort:PARameter`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``MMEMory:IMPort:PARameter?`` query.
            - Using the ``.verify(value)`` method will send the ``MMEMory:IMPort:PARameter?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.frequency``: The ``MMEMory:IMPort:PARameter:FREQuency`` command tree.
            - ``.level``: The ``MMEMory:IMPort:PARameter:LEVel`` command tree.
            - ``.normalize``: The ``MMEMory:IMPort:PARameter:NORMalize`` command.
            - ``.resampling``: The ``MMEMory:IMPort:PARameter:RESampling`` command tree.
        """
        return self._parameter


class MmemoryExport(SCPICmdWrite):
    r"""The ``MMEMory:EXPort`` command.

    **Description:**
        - This command exports a waveform file from the arbitrary waveform generator setup. The
          supported file formats are: TXT - Text file with analog data TXT8 - Text file with 8-bit
          DAC resolution TXT10 - Text file with 10-bit DAC resolution TXT14 - Text file with 14-bit
          DAC resolution (AWG5000 Series only) WFM - AWG400/AWG500/AWG600/AWG700 Series waveform

    **Usage:**
        - Using the ``.write(value)`` method will send the ``MMEMory:EXPort value`` command.

    **SCPI Syntax:**

    ::

        - MMEMory:EXPort <wfm_name>,<filename>,<type>

    **Info:**
        - ``<wfm_name>,<filename>,<type>``
        - ``<wfm_name>`` ::=<string>.
        - ``<filename>`` ::=<string>.
        - ``<type>`` = {TXT\|TXT8\|TXT10\|TXT14\|WFM}.
    """


class MmemoryDelete(SCPICmdWrite):
    """The ``MMEMory:DELete`` command.

    **Description:**
        - This command deletes a file or directory from the AWG's hard disk. When used on a
          directory, this command succeeds only if the directory is empty.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``MMEMory:DELete value`` command.

    **SCPI Syntax:**

    ::

        - MMEMory:DELete <file_name>[,<msus>]
    """


class MmemoryData(SCPICmdWrite):
    """The ``MMEMory:DATA`` command.

    **Description:**
        - This command and query sets or returns block data to/from the file in the current mass
          storage device. This command has a limit of 650,000,000 bytes of data. If this limit is
          insufficient, consider the following alternatives: Use a more efficient file encoding (WFM
          or PAT) when sending data. Use instrument commands for direct control
          (``WLIST:WAVEFORM:DATA``, FREQ, VOLT, and so on). Use Ethernet (ftp, http, or file
          sharing) to transfer the file.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``MMEMory:DATA value`` command.

    **SCPI Syntax:**

    ::

        - MMEMory:DATA <file_name>,<block_data>? <file_name>

    **Info:**
        - ``<file_name>`` , ``<block_data>``.
    """


class MmemoryCdirectory(SCPICmdWrite, SCPICmdRead):
    """The ``MMEMory:CDIRectory`` command.

    **Description:**
        - This command changes the current working directory in the mass storage system.

    **Usage:**
        - Using the ``.query()`` method will send the ``MMEMory:CDIRectory?`` query.
        - Using the ``.verify(value)`` method will send the ``MMEMory:CDIRectory?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MMEMory:CDIRectory value`` command.

    **SCPI Syntax:**

    ::

        - MMEMory:CDIRectory [<directory_name>]
        - MMEMory:CDIRectory?
    """


class MmemoryCatalog(SCPICmdReadWithArguments):
    """The ``MMEMory:CATalog`` command.

    **Description:**
        - This command returns the current contents and state of the mass storage media.

    **Usage:**
        - Using the ``.query(argument)`` method will send the ``MMEMory:CATalog? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the ``MMEMory:CATalog? argument``
          query and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - MMEMory:CATalog? [<msus>]
    """


#  pylint: disable=too-many-instance-attributes
class Mmemory(SCPICmdRead):
    """The ``MMEMory`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``MMEMory?`` query.
        - Using the ``.verify(value)`` method will send the ``MMEMory?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.catalog``: The ``MMEMory:CATalog`` command.
        - ``.cdirectory``: The ``MMEMory:CDIRectory`` command.
        - ``.data``: The ``MMEMory:DATA`` command.
        - ``.delete``: The ``MMEMory:DELete`` command.
        - ``.export``: The ``MMEMory:EXPort`` command.
        - ``.import``: The ``MMEMory:IMPort`` command.
        - ``.mdirectory``: The ``MMEMory:MDIRectory`` command.
        - ``.msis``: The ``MMEMory:MSIS`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "MMEMory") -> None:
        super().__init__(device, cmd_syntax)
        self._catalog = MmemoryCatalog(device, f"{self._cmd_syntax}:CATalog")
        self._cdirectory = MmemoryCdirectory(device, f"{self._cmd_syntax}:CDIRectory")
        self._data = MmemoryData(device, f"{self._cmd_syntax}:DATA")
        self._delete = MmemoryDelete(device, f"{self._cmd_syntax}:DELete")
        self._export = MmemoryExport(device, f"{self._cmd_syntax}:EXPort")
        self._import = MmemoryImport(device, f"{self._cmd_syntax}:IMPort")
        self._mdirectory = MmemoryMdirectory(device, f"{self._cmd_syntax}:MDIRectory")
        self._msis = MmemoryMsis(device, f"{self._cmd_syntax}:MSIS")

    @property
    def catalog(self) -> MmemoryCatalog:
        """Return the ``MMEMory:CATalog`` command.

        **Description:**
            - This command returns the current contents and state of the mass storage media.

        **Usage:**
            - Using the ``.query(argument)`` method will send the ``MMEMory:CATalog? argument``
              query.
            - Using the ``.verify(argument, value)`` method will send the
              ``MMEMory:CATalog? argument`` query and raise an AssertionError if the returned value
              does not match ``value``.

        **SCPI Syntax:**

        ::

            - MMEMory:CATalog? [<msus>]
        """
        return self._catalog

    @property
    def cdirectory(self) -> MmemoryCdirectory:
        """Return the ``MMEMory:CDIRectory`` command.

        **Description:**
            - This command changes the current working directory in the mass storage system.

        **Usage:**
            - Using the ``.query()`` method will send the ``MMEMory:CDIRectory?`` query.
            - Using the ``.verify(value)`` method will send the ``MMEMory:CDIRectory?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MMEMory:CDIRectory value`` command.

        **SCPI Syntax:**

        ::

            - MMEMory:CDIRectory [<directory_name>]
            - MMEMory:CDIRectory?
        """
        return self._cdirectory

    @property
    def data(self) -> MmemoryData:
        """Return the ``MMEMory:DATA`` command.

        **Description:**
            - This command and query sets or returns block data to/from the file in the current mass
              storage device. This command has a limit of 650,000,000 bytes of data. If this limit
              is insufficient, consider the following alternatives: Use a more efficient file
              encoding (WFM or PAT) when sending data. Use instrument commands for direct control
              (``WLIST:WAVEFORM:DATA``, FREQ, VOLT, and so on). Use Ethernet (ftp, http, or file
              sharing) to transfer the file.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``MMEMory:DATA value`` command.

        **SCPI Syntax:**

        ::

            - MMEMory:DATA <file_name>,<block_data>? <file_name>

        **Info:**
            - ``<file_name>`` , ``<block_data>``.
        """
        return self._data

    @property
    def delete(self) -> MmemoryDelete:
        """Return the ``MMEMory:DELete`` command.

        **Description:**
            - This command deletes a file or directory from the AWG's hard disk. When used on a
              directory, this command succeeds only if the directory is empty.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``MMEMory:DELete value`` command.

        **SCPI Syntax:**

        ::

            - MMEMory:DELete <file_name>[,<msus>]
        """
        return self._delete

    @property
    def export(self) -> MmemoryExport:
        r"""Return the ``MMEMory:EXPort`` command.

        **Description:**
            - This command exports a waveform file from the arbitrary waveform generator setup. The
              supported file formats are: TXT - Text file with analog data TXT8 - Text file with
              8-bit DAC resolution TXT10 - Text file with 10-bit DAC resolution TXT14 - Text file
              with 14-bit DAC resolution (AWG5000 Series only) WFM - AWG400/AWG500/AWG600/AWG700
              Series waveform

        **Usage:**
            - Using the ``.write(value)`` method will send the ``MMEMory:EXPort value`` command.

        **SCPI Syntax:**

        ::

            - MMEMory:EXPort <wfm_name>,<filename>,<type>

        **Info:**
            - ``<wfm_name>,<filename>,<type>``
            - ``<wfm_name>`` ::=<string>.
            - ``<filename>`` ::=<string>.
            - ``<type>`` = {TXT\|TXT8\|TXT10\|TXT14\|WFM}.
        """
        return self._export

    @property
    def import_(self) -> MmemoryImport:
        r"""Return the ``MMEMory:IMPort`` command.

        **Description:**
            - This command imports a file into the arbitrary waveform generator's setup as a
              waveform. The supported file formats are: ISF - TDS3000 and DPO4000 waveform format
              TDS - TDS5000/TDS6000/TDS7000, DPO7000/DPO70000/DSA70000 Series waveform TXT - Text
              file with analog data TXT8 - Text file with 8-bit DAC resolution TXT10 - Text file
              with 10-bit DAC resolution TXT14 - Text file with 14-bit DAC resolution WFM -
              AWG400/AWG500/AWG600/AWG700 Series waveform PAT - AWG400/AWG500/AWG600/AWG700 Series
              pattern file TFW - AFG3000 Series waveform file format IQT - RSA3000 Series waveform
              file format TIQ - RSA6000 Series waveform file format

        **Usage:**
            - Using the ``.write(value)`` method will send the ``MMEMory:IMPort value`` command.

        **SCPI Syntax:**

        ::

            - MMEMory:IMPort <wfm_name>,<filename>,<type>

        **Info:**
            - ``<wfm_name>,<filename>,<type>``
            - ``<wfm_name>`` ::=<string>.
            - ``<filename>`` ::=<string>.
            - ``<type>`` = {ISF\|TDS\|TXT\|TXT8\|TXT10\|TXT14\|WFM\|PAT\|TFW}.

        Sub-properties:
            - ``.parameter``: The ``MMEMory:IMPort:PARameter`` command tree.
        """
        return self._import

    @property
    def mdirectory(self) -> MmemoryMdirectory:
        """Return the ``MMEMory:MDIRectory`` command.

        **Description:**
            - This command creates a directory in the mass storage system. If the specified
              directory is locked in the mass storage system, this command causes an error.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``MMEMory:MDIRectory value`` command.

        **SCPI Syntax:**

        ::

            - MMEMory:MDIRectory <directory_name>
        """
        return self._mdirectory

    @property
    def msis(self) -> MmemoryMsis:
        """Return the ``MMEMory:MSIS`` command.

        **Description:**
            - This command selects or returns a mass storage device used by all MMEMory commands.
              <msus> specifies a drive using a drive letter. The drive letter can represent hard
              disk drives, network drives, external DVD/CD-RW drives, or USB memory.

        **Usage:**
            - Using the ``.query()`` method will send the ``MMEMory:MSIS?`` query.
            - Using the ``.verify(value)`` method will send the ``MMEMory:MSIS?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MMEMory:MSIS value`` command.

        **SCPI Syntax:**

        ::

            - MMEMory:MSIS [<msus>]
            - MMEMory:MSIS?
        """
        return self._msis



"""The abort commands module.

These commands are used in the following models:
AWG5200

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - ABORt [{ATRigger|BTRigger}]
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Abort(SCPICmdWrite):
    """The ``ABORt`` command.

    **Description:**
        - This command stops waveform playout when the Run Mode is set to Gated. This is equivalent
          to releasing the Force Trig button on the front panel when the instrument is in gated
          mode.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``ABORt value`` command.

    **SCPI Syntax:**

    ::

        - ABORt [{ATRigger|BTRigger}]
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "ABORt") -> None:
        super().__init__(device, cmd_syntax)



"""The awgcontrol commands module.

These commands are used in the following models:
AWG5200

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - AWGControl:ARSettings {0|1|ON|OFF}
    - AWGControl:CLOCk:DRATe <NR1>
    - AWGControl:CLOCk:DRATe?
    - AWGControl:CLOCk:PHASe:ADJust <NR1>
    - AWGControl:CLOCk:PHASe:ADJust?
    - AWGControl:CLOCk:SOURce {INTernal|EXTernal|EFIXed|EVARiable}
    - AWGControl:CLOCk:SOURce?
    - AWGControl:COMPile <filename>
    - AWGControl:CONFigure:CNUMber?
    - AWGControl:PJUMp:JSTRobe {0|1|OFF|ON}
    - AWGControl:PJUMp:JSTRobe?
    - AWGControl:PJUMp:SEDGe {FALLing|RISing}
    - AWGControl:PJUMp:SEDGe?
    - AWGControl:RMODe {CONTinuous|TRIGgered|GATed}
    - AWGControl:RMODe?
    - AWGControl:RSTate?
    - AWGControl:RUN:IMMediate
    - AWGControl:SNAMe?
    - AWGControl:SREStore <filepath>[,<msus>]
    - AWGControl:SSAVe <filepath>[,<msus>]
    - AWGControl:STOP:IMMediate
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class AwgcontrolStopImmediate(SCPICmdWriteNoArguments):
    """The ``AWGControl:STOP:IMMediate`` command.

    **Description:**
        - This command stops the output of a waveform or a sequence.

    **Usage:**
        - Using the ``.write()`` method will send the ``AWGControl:STOP:IMMediate`` command.

    **SCPI Syntax:**

    ::

        - AWGControl:STOP:IMMediate
    """


class AwgcontrolStop(SCPICmdRead):
    """The ``AWGControl:STOP`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:STOP?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:STOP?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.immediate``: The ``AWGControl:STOP:IMMediate`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._immediate = AwgcontrolStopImmediate(device, f"{self._cmd_syntax}:IMMediate")

    @property
    def immediate(self) -> AwgcontrolStopImmediate:
        """Return the ``AWGControl:STOP:IMMediate`` command.

        **Description:**
            - This command stops the output of a waveform or a sequence.

        **Usage:**
            - Using the ``.write()`` method will send the ``AWGControl:STOP:IMMediate`` command.

        **SCPI Syntax:**

        ::

            - AWGControl:STOP:IMMediate
        """
        return self._immediate


class AwgcontrolSsave(SCPICmdWrite):
    """The ``AWGControl:SSAVe`` command.

    **Description:**
        - This command saves the AWG's setup with waveforms.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``AWGControl:SSAVe value`` command.

    **SCPI Syntax:**

    ::

        - AWGControl:SSAVe <filepath>[,<msus>]
    """


class AwgcontrolSrestore(SCPICmdWrite):
    """The ``AWGControl:SREStore`` command.

    **Description:**
        - This command opens a setup file into the AWG's setup memory.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``AWGControl:SREStore value`` command.

    **SCPI Syntax:**

    ::

        - AWGControl:SREStore <filepath>[,<msus>]
    """


class AwgcontrolSname(SCPICmdRead):
    """The ``AWGControl:SNAMe`` command.

    **Description:**
        - This command returns the AWG's most recently saved setup location. The response contains
          the full path for the file, including the disk drive letter (msus or, mass storage unit
          specifier).

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:SNAMe?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:SNAMe?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - AWGControl:SNAMe?
    """


class AwgcontrolRunImmediate(SCPICmdWriteNoArguments):
    """The ``AWGControl:RUN:IMMediate`` command.

    **Description:**
        - This command initiates the output of a waveform or sequence. This is equivalent to pushing
          the play button on the front-panel or display. The AWG can be put in the run state only
          when waveforms or sequences are assigned to channels.

    **Usage:**
        - Using the ``.write()`` method will send the ``AWGControl:RUN:IMMediate`` command.

    **SCPI Syntax:**

    ::

        - AWGControl:RUN:IMMediate
    """


class AwgcontrolRun(SCPICmdRead):
    """The ``AWGControl:RUN`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:RUN?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:RUN?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.immediate``: The ``AWGControl:RUN:IMMediate`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._immediate = AwgcontrolRunImmediate(device, f"{self._cmd_syntax}:IMMediate")

    @property
    def immediate(self) -> AwgcontrolRunImmediate:
        """Return the ``AWGControl:RUN:IMMediate`` command.

        **Description:**
            - This command initiates the output of a waveform or sequence. This is equivalent to
              pushing the play button on the front-panel or display. The AWG can be put in the run
              state only when waveforms or sequences are assigned to channels.

        **Usage:**
            - Using the ``.write()`` method will send the ``AWGControl:RUN:IMMediate`` command.

        **SCPI Syntax:**

        ::

            - AWGControl:RUN:IMMediate
        """
        return self._immediate


class AwgcontrolRstate(SCPICmdRead):
    """The ``AWGControl:RSTate`` command.

    **Description:**
        - This command returns the run state of the AWG.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:RSTate?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:RSTate?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - AWGControl:RSTate?
    """


class AwgcontrolRmode(SCPICmdWrite, SCPICmdRead):
    """The ``AWGControl:RMODe`` command.

    **Description:**
        - This command sets or returns the run mode of the AWG for Channel 1.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:RMODe?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:RMODe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AWGControl:RMODe value`` command.

    **SCPI Syntax:**

    ::

        - AWGControl:RMODe {CONTinuous|TRIGgered|GATed}
        - AWGControl:RMODe?

    **Info:**
        - ``CONTinuous`` sets the Run Mode to Continuous (not waiting for a trigger event).
          TRIGgered sets the Run Mode to Triggered, waiting for a trigger event. One waveform play
          out cycle completes, then play out stops, waiting for the next trigger event. GATed sets
          the Run Mode to only playout a waveform while the trigger is enabled.
        - ``*RST`` sets this to CONT.
    """


class AwgcontrolPjumpSedge(SCPICmdWrite, SCPICmdRead):
    """The ``AWGControl:PJUMp:SEDGe`` command.

    **Description:**
        - This command sets or returns the active Strobe Edge to use for Pattern Jump when Pattern
          Jump is enabled for Sequencing.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:PJUMp:SEDGe?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:PJUMp:SEDGe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AWGControl:PJUMp:SEDGe value`` command.

    **SCPI Syntax:**

    ::

        - AWGControl:PJUMp:SEDGe {FALLing|RISing}
        - AWGControl:PJUMp:SEDGe?

    **Info:**
        - ``FALLing`` sets the falling edge of the strobe signal to the active edge. 256 input
          patterns are available.
        - ``RISing`` sets the rising edge of the strobe signal to the active edge. 256 input
          patterns are available.
    """


class AwgcontrolPjumpJstrobe(SCPICmdWrite, SCPICmdRead):
    """The ``AWGControl:PJUMp:JSTRobe`` command.

    **Description:**
        - This command sets or returns if the pattern jump is set (enabled or disabled) to always
          occur on the strobe signal. With this setting disabled, the pattern jump requires a strobe
          signal and a pattern address change to initiate a jump. With this setting enabled, the
          pattern jump disregards the pattern address change condition, causing the jump to always
          occur on the strobe signal.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:PJUMp:JSTRobe?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:PJUMp:JSTRobe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AWGControl:PJUMp:JSTRobe value``
          command.

    **SCPI Syntax:**

    ::

        - AWGControl:PJUMp:JSTRobe {0|1|OFF|ON}
        - AWGControl:PJUMp:JSTRobe?
    """


class AwgcontrolPjump(SCPICmdRead):
    """The ``AWGControl:PJUMp`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:PJUMp?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:PJUMp?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.jstrobe``: The ``AWGControl:PJUMp:JSTRobe`` command.
        - ``.sedge``: The ``AWGControl:PJUMp:SEDGe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._jstrobe = AwgcontrolPjumpJstrobe(device, f"{self._cmd_syntax}:JSTRobe")
        self._sedge = AwgcontrolPjumpSedge(device, f"{self._cmd_syntax}:SEDGe")

    @property
    def jstrobe(self) -> AwgcontrolPjumpJstrobe:
        """Return the ``AWGControl:PJUMp:JSTRobe`` command.

        **Description:**
            - This command sets or returns if the pattern jump is set (enabled or disabled) to
              always occur on the strobe signal. With this setting disabled, the pattern jump
              requires a strobe signal and a pattern address change to initiate a jump. With this
              setting enabled, the pattern jump disregards the pattern address change condition,
              causing the jump to always occur on the strobe signal.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:PJUMp:JSTRobe?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:PJUMp:JSTRobe?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AWGControl:PJUMp:JSTRobe value``
              command.

        **SCPI Syntax:**

        ::

            - AWGControl:PJUMp:JSTRobe {0|1|OFF|ON}
            - AWGControl:PJUMp:JSTRobe?
        """
        return self._jstrobe

    @property
    def sedge(self) -> AwgcontrolPjumpSedge:
        """Return the ``AWGControl:PJUMp:SEDGe`` command.

        **Description:**
            - This command sets or returns the active Strobe Edge to use for Pattern Jump when
              Pattern Jump is enabled for Sequencing.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:PJUMp:SEDGe?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:PJUMp:SEDGe?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AWGControl:PJUMp:SEDGe value``
              command.

        **SCPI Syntax:**

        ::

            - AWGControl:PJUMp:SEDGe {FALLing|RISing}
            - AWGControl:PJUMp:SEDGe?

        **Info:**
            - ``FALLing`` sets the falling edge of the strobe signal to the active edge. 256 input
              patterns are available.
            - ``RISing`` sets the rising edge of the strobe signal to the active edge. 256 input
              patterns are available.
        """
        return self._sedge


class AwgcontrolConfigureCnumber(SCPICmdRead):
    """The ``AWGControl:CONFigure:CNUMber`` command.

    **Description:**
        - This command returns the number of channels available on the AWG.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:CONFigure:CNUMber?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:CONFigure:CNUMber?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - AWGControl:CONFigure:CNUMber?
    """


class AwgcontrolConfigure(SCPICmdRead):
    """The ``AWGControl:CONFigure`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:CONFigure?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:CONFigure?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.cnumber``: The ``AWGControl:CONFigure:CNUMber`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._cnumber = AwgcontrolConfigureCnumber(device, f"{self._cmd_syntax}:CNUMber")

    @property
    def cnumber(self) -> AwgcontrolConfigureCnumber:
        """Return the ``AWGControl:CONFigure:CNUMber`` command.

        **Description:**
            - This command returns the number of channels available on the AWG.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:CONFigure:CNUMber?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:CONFigure:CNUMber?``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - AWGControl:CONFigure:CNUMber?
        """
        return self._cnumber


class AwgcontrolCompile(SCPICmdWrite):
    """The ``AWGControl:COMPile`` command.

    **Description:**
        - This command compiles an equation file and imports the waveforms (created by the equation
          file) into the arbitrary waveform generator.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``AWGControl:COMPile value`` command.

    **SCPI Syntax:**

    ::

        - AWGControl:COMPile <filename>
    """


class AwgcontrolClockSource(SCPICmdWrite, SCPICmdRead):
    """The ``AWGControl:CLOCk:SOURce`` command.

    **Description:**
        - This command sets or returns the source of the clock.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:CLOCk:SOURce?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:SOURce?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AWGControl:CLOCk:SOURce value``
          command.

    **SCPI Syntax:**

    ::

        - AWGControl:CLOCk:SOURce {INTernal|EXTernal|EFIXed|EVARiable}
        - AWGControl:CLOCk:SOURce?

    **Info:**
        - ``INTernal`` - clock signal is generated internally and the reference frequency is derived
          by the internal oscillator. EFIXed - clock is generated internally and the reference
          frequency is derived from a fixed 10 MHz reference supplied at the Reference In connector.
          EVARiable - clock is generated internally and the reference frequency is derived from a
          variable reference supplied at the Reference In connector. EXTernal - clock signal
          supplied by the Clock In connector and the reference frequency is derived from the
          internal precision oscillator.
        - ``*RST`` sets this to INT.
    """


class AwgcontrolClockPhaseAdjust(SCPICmdWrite, SCPICmdRead):
    """The ``AWGControl:CLOCk:PHASe:ADJust`` command.

    **Description:**
        - This command sets or returns the internal clock phase adjustment of the AWG.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:CLOCk:PHASe:ADJust?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:PHASe:ADJust?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AWGControl:CLOCk:PHASe:ADJust value``
          command.

    **SCPI Syntax:**

    ::

        - AWGControl:CLOCk:PHASe:ADJust <NR1>
        - AWGControl:CLOCk:PHASe:ADJust?
    """


class AwgcontrolClockPhase(SCPICmdRead):
    """The ``AWGControl:CLOCk:PHASe`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:CLOCk:PHASe?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:PHASe?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.adjust``: The ``AWGControl:CLOCk:PHASe:ADJust`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._adjust = AwgcontrolClockPhaseAdjust(device, f"{self._cmd_syntax}:ADJust")

    @property
    def adjust(self) -> AwgcontrolClockPhaseAdjust:
        """Return the ``AWGControl:CLOCk:PHASe:ADJust`` command.

        **Description:**
            - This command sets or returns the internal clock phase adjustment of the AWG.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:CLOCk:PHASe:ADJust?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:PHASe:ADJust?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``AWGControl:CLOCk:PHASe:ADJust value`` command.

        **SCPI Syntax:**

        ::

            - AWGControl:CLOCk:PHASe:ADJust <NR1>
            - AWGControl:CLOCk:PHASe:ADJust?
        """
        return self._adjust


class AwgcontrolClockDrate(SCPICmdWrite, SCPICmdRead):
    """The ``AWGControl:CLOCk:DRATe`` command.

    **Description:**
        - This command sets or returns the divider rate for the external clock.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:CLOCk:DRATe?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:DRATe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AWGControl:CLOCk:DRATe value`` command.

    **SCPI Syntax:**

    ::

        - AWGControl:CLOCk:DRATe <NR1>
        - AWGControl:CLOCk:DRATe?

    **Info:**
        - ``*RST`` sets this to 1.
    """


class AwgcontrolClock(SCPICmdRead):
    """The ``AWGControl:CLOCk`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl:CLOCk?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.phase``: The ``AWGControl:CLOCk:PHASe`` command tree.
        - ``.drate``: The ``AWGControl:CLOCk:DRATe`` command.
        - ``.source``: The ``AWGControl:CLOCk:SOURce`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._phase = AwgcontrolClockPhase(device, f"{self._cmd_syntax}:PHASe")
        self._drate = AwgcontrolClockDrate(device, f"{self._cmd_syntax}:DRATe")
        self._source = AwgcontrolClockSource(device, f"{self._cmd_syntax}:SOURce")

    @property
    def phase(self) -> AwgcontrolClockPhase:
        """Return the ``AWGControl:CLOCk:PHASe`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:CLOCk:PHASe?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:PHASe?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.adjust``: The ``AWGControl:CLOCk:PHASe:ADJust`` command.
        """
        return self._phase

    @property
    def drate(self) -> AwgcontrolClockDrate:
        """Return the ``AWGControl:CLOCk:DRATe`` command.

        **Description:**
            - This command sets or returns the divider rate for the external clock.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:CLOCk:DRATe?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:DRATe?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AWGControl:CLOCk:DRATe value``
              command.

        **SCPI Syntax:**

        ::

            - AWGControl:CLOCk:DRATe <NR1>
            - AWGControl:CLOCk:DRATe?

        **Info:**
            - ``*RST`` sets this to 1.
        """
        return self._drate

    @property
    def source(self) -> AwgcontrolClockSource:
        """Return the ``AWGControl:CLOCk:SOURce`` command.

        **Description:**
            - This command sets or returns the source of the clock.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:CLOCk:SOURce?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:SOURce?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AWGControl:CLOCk:SOURce value``
              command.

        **SCPI Syntax:**

        ::

            - AWGControl:CLOCk:SOURce {INTernal|EXTernal|EFIXed|EVARiable}
            - AWGControl:CLOCk:SOURce?

        **Info:**
            - ``INTernal`` - clock signal is generated internally and the reference frequency is
              derived by the internal oscillator. EFIXed - clock is generated internally and the
              reference frequency is derived from a fixed 10 MHz reference supplied at the Reference
              In connector. EVARiable - clock is generated internally and the reference frequency is
              derived from a variable reference supplied at the Reference In connector. EXTernal -
              clock signal supplied by the Clock In connector and the reference frequency is derived
              from the internal precision oscillator.
            - ``*RST`` sets this to INT.
        """
        return self._source


class AwgcontrolArsettings(SCPICmdWrite):
    """The ``AWGControl:ARSettings`` command.

    **Description:**
        - This command sets or returns the state (enabled or disabled) of whether or not to apply
          the recommended settings of waveforms and sequences when they are assigned to a channel.
          When enabled, the system attempts to use the waveform's recommended settings (sample rate,
          amplitude, and offset) when the waveform is assigned to a channel. This includes waveforms
          within sequence tracks assigned to a channel. Recommended settings are defined as part the
          waveform properties and sequence properties.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``AWGControl:ARSettings value`` command.

    **SCPI Syntax:**

    ::

        - AWGControl:ARSettings {0|1|ON|OFF}
    """


#  pylint: disable=too-many-instance-attributes
class Awgcontrol(SCPICmdRead):
    """The ``AWGControl`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AWGControl?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.arsettings``: The ``AWGControl:ARSettings`` command.
        - ``.clock``: The ``AWGControl:CLOCk`` command tree.
        - ``.compile``: The ``AWGControl:COMPile`` command.
        - ``.configure``: The ``AWGControl:CONFigure`` command tree.
        - ``.pjump``: The ``AWGControl:PJUMp`` command tree.
        - ``.rmode``: The ``AWGControl:RMODe`` command.
        - ``.rstate``: The ``AWGControl:RSTate`` command.
        - ``.run``: The ``AWGControl:RUN`` command tree.
        - ``.sname``: The ``AWGControl:SNAMe`` command.
        - ``.srestore``: The ``AWGControl:SREStore`` command.
        - ``.ssave``: The ``AWGControl:SSAVe`` command.
        - ``.stop``: The ``AWGControl:STOP`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "AWGControl") -> None:
        super().__init__(device, cmd_syntax)
        self._arsettings = AwgcontrolArsettings(device, f"{self._cmd_syntax}:ARSettings")
        self._compile = AwgcontrolCompile(device, f"{self._cmd_syntax}:COMPile")
        self._configure = AwgcontrolConfigure(device, f"{self._cmd_syntax}:CONFigure")
        self._pjump = AwgcontrolPjump(device, f"{self._cmd_syntax}:PJUMp")
        self._rmode = AwgcontrolRmode(device, f"{self._cmd_syntax}:RMODe")
        self._rstate = AwgcontrolRstate(device, f"{self._cmd_syntax}:RSTate")
        self._run = AwgcontrolRun(device, f"{self._cmd_syntax}:RUN")
        self._sname = AwgcontrolSname(device, f"{self._cmd_syntax}:SNAMe")
        self._srestore = AwgcontrolSrestore(device, f"{self._cmd_syntax}:SREStore")
        self._ssave = AwgcontrolSsave(device, f"{self._cmd_syntax}:SSAVe")
        self._stop = AwgcontrolStop(device, f"{self._cmd_syntax}:STOP")
        self._clock = AwgcontrolClock(device, f"{self._cmd_syntax}:CLOCk")

    @property
    def arsettings(self) -> AwgcontrolArsettings:
        """Return the ``AWGControl:ARSettings`` command.

        **Description:**
            - This command sets or returns the state (enabled or disabled) of whether or not to
              apply the recommended settings of waveforms and sequences when they are assigned to a
              channel. When enabled, the system attempts to use the waveform's recommended settings
              (sample rate, amplitude, and offset) when the waveform is assigned to a channel. This
              includes waveforms within sequence tracks assigned to a channel. Recommended settings
              are defined as part the waveform properties and sequence properties.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``AWGControl:ARSettings value``
              command.

        **SCPI Syntax:**

        ::

            - AWGControl:ARSettings {0|1|ON|OFF}
        """
        return self._arsettings

    @property
    def compile(self) -> AwgcontrolCompile:
        """Return the ``AWGControl:COMPile`` command.

        **Description:**
            - This command compiles an equation file and imports the waveforms (created by the
              equation file) into the arbitrary waveform generator.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``AWGControl:COMPile value`` command.

        **SCPI Syntax:**

        ::

            - AWGControl:COMPile <filename>
        """
        return self._compile

    @property
    def configure(self) -> AwgcontrolConfigure:
        """Return the ``AWGControl:CONFigure`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:CONFigure?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:CONFigure?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.cnumber``: The ``AWGControl:CONFigure:CNUMber`` command.
        """
        return self._configure

    @property
    def pjump(self) -> AwgcontrolPjump:
        """Return the ``AWGControl:PJUMp`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:PJUMp?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:PJUMp?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.jstrobe``: The ``AWGControl:PJUMp:JSTRobe`` command.
            - ``.sedge``: The ``AWGControl:PJUMp:SEDGe`` command.
        """
        return self._pjump

    @property
    def rmode(self) -> AwgcontrolRmode:
        """Return the ``AWGControl:RMODe`` command.

        **Description:**
            - This command sets or returns the run mode of the AWG for Channel 1.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:RMODe?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:RMODe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AWGControl:RMODe value`` command.

        **SCPI Syntax:**

        ::

            - AWGControl:RMODe {CONTinuous|TRIGgered|GATed}
            - AWGControl:RMODe?

        **Info:**
            - ``CONTinuous`` sets the Run Mode to Continuous (not waiting for a trigger event).
              TRIGgered sets the Run Mode to Triggered, waiting for a trigger event. One waveform
              play out cycle completes, then play out stops, waiting for the next trigger event.
              GATed sets the Run Mode to only playout a waveform while the trigger is enabled.
            - ``*RST`` sets this to CONT.
        """
        return self._rmode

    @property
    def rstate(self) -> AwgcontrolRstate:
        """Return the ``AWGControl:RSTate`` command.

        **Description:**
            - This command returns the run state of the AWG.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:RSTate?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:RSTate?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - AWGControl:RSTate?
        """
        return self._rstate

    @property
    def run(self) -> AwgcontrolRun:
        """Return the ``AWGControl:RUN`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:RUN?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:RUN?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.immediate``: The ``AWGControl:RUN:IMMediate`` command.
        """
        return self._run

    @property
    def sname(self) -> AwgcontrolSname:
        """Return the ``AWGControl:SNAMe`` command.

        **Description:**
            - This command returns the AWG's most recently saved setup location. The response
              contains the full path for the file, including the disk drive letter (msus or, mass
              storage unit specifier).

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:SNAMe?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:SNAMe?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - AWGControl:SNAMe?
        """
        return self._sname

    @property
    def srestore(self) -> AwgcontrolSrestore:
        """Return the ``AWGControl:SREStore`` command.

        **Description:**
            - This command opens a setup file into the AWG's setup memory.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``AWGControl:SREStore value``
              command.

        **SCPI Syntax:**

        ::

            - AWGControl:SREStore <filepath>[,<msus>]
        """
        return self._srestore

    @property
    def ssave(self) -> AwgcontrolSsave:
        """Return the ``AWGControl:SSAVe`` command.

        **Description:**
            - This command saves the AWG's setup with waveforms.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``AWGControl:SSAVe value`` command.

        **SCPI Syntax:**

        ::

            - AWGControl:SSAVe <filepath>[,<msus>]
        """
        return self._ssave

    @property
    def stop(self) -> AwgcontrolStop:
        """Return the ``AWGControl:STOP`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:STOP?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:STOP?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.immediate``: The ``AWGControl:STOP:IMMediate`` command.
        """
        return self._stop

    @property
    def clock(self) -> AwgcontrolClock:
        """Return the ``AWGControl:CLOCk`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``AWGControl:CLOCk?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.phase``: The ``AWGControl:CLOCk:PHASe`` command tree.
            - ``.drate``: The ``AWGControl:CLOCk:DRATe`` command.
            - ``.source``: The ``AWGControl:CLOCk:SOURce`` command.
        """
        return self._clock



"""SMU Model 2651A device driver module."""
from tm_devices.commands import SMU2651AMixin
from tm_devices.drivers.pi.source_measure_units.smu2600.smu2650a import SMU2650A


class SMU2651A(SMU2651AMixin, SMU2650A):
    """SMU2651A device driver."""



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The channel commands module.

These commands are used in the following models:
DMM6500

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - channel.close()
    - channel.getclose()
    - channel.getcount()
    - channel.getdelay()
    - channel.getdmm()
    - channel.getlabel()
    - channel.getstate()
    - channel.gettype()
    - channel.multiple.close()
    - channel.multiple.open()
    - channel.open()
    - channel.setdelay()
    - channel.setdmm()
    - channel.setlabel()
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class ChannelMultiple(BaseTSPCmd):
    """The ``channel.multiple`` command tree.

    Properties/methods:
        - ``.close()``: The ``channel.multiple.close()`` function.
        - ``.open()``: The ``channel.multiple.open()`` function.
    """

    def close(self, channel_list: str) -> None:
        """Run the ``channel.multiple.close()`` function.

        **Description:**
            - This function closes the listed channels without affecting any other channels.

        **TSP Syntax:**

        ::

            - channel.multiple.close()

        Args:
            channel_list: The list of channels to close.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.close("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.close()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def open(self, channel_list: str) -> None:
        """Run the ``channel.multiple.open()`` function.

        **Description:**
            - This function opens the channels in the channel list without affecting any others.

        **TSP Syntax:**

        ::

            - channel.multiple.open()

        Args:
            channel_list: A list of the channels to open.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.open("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.open()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Channel(BaseTSPCmd):
    """The ``channel`` command tree.

    Properties/methods:
        - ``.close()``: The ``channel.close()`` function.
        - ``.getclose()``: The ``channel.getclose()`` function.
        - ``.getcount()``: The ``channel.getcount()`` function.
        - ``.getdelay()``: The ``channel.getdelay()`` function.
        - ``.getdmm()``: The ``channel.getdmm()`` function.
        - ``.getlabel()``: The ``channel.getlabel()`` function.
        - ``.getstate()``: The ``channel.getstate()`` function.
        - ``.gettype()``: The ``channel.gettype()`` function.
        - ``.multiple``: The ``channel.multiple`` command tree.
        - ``.open()``: The ``channel.open()`` function.
        - ``.setdelay()``: The ``channel.setdelay()`` function.
        - ``.setdmm()``: The ``channel.setdmm()`` function.
        - ``.setlabel()``: The ``channel.setlabel()`` function.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "channel") -> None:
        super().__init__(device, cmd_syntax)
        self._multiple = ChannelMultiple(device, f"{self._cmd_syntax}.multiple")

    @property
    def multiple(self) -> ChannelMultiple:
        """Return the ``channel.multiple`` command tree.

        Sub-properties/methods:
            - ``.close()``: The ``channel.multiple.close()`` function.
            - ``.open()``: The ``channel.multiple.open()`` function.
        """
        return self._multiple

    def close(self, channel_list: str) -> None:
        """Run the ``channel.close()`` function.

        **Description:**
            - This function closes the channels and channel pairs that are specified by the channel
              list parameter.

        **TSP Syntax:**

        ::

            - channel.close()

        Args:
            channel_list: A string that lists the channels and channel pairs to close.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.close("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.close()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getclose(self, channel_list: Optional[str] = None) -> str:
        """Run the ``channel.getclose()`` function.

        **Description:**
            - This function queries for the closed channels indicated by the channel list parameter.

        **TSP Syntax:**

        ::

            - channel.getclose()

        Args:
            channel_list (optional): A string representing the channels and channel patterns that
                will be queried.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (f'"{channel_list}"' if channel_list is not None else None,)
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.getclose({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getclose()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getcount(self, channel_list: Optional[str] = None) -> str:
        """Run the ``channel.getcount()`` function.

        **Description:**
            - This function returns the number of times the relays have been closed for the
              specified channels.

        **TSP Syntax:**

        ::

            - channel.getcount()

        Args:
            channel_list (optional): A string listing the items to query, which can include.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (f'"{channel_list}"' if channel_list is not None else None,)
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.getcount({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getcount()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getdelay(self, channel_list: str) -> str:
        """Run the ``channel.getdelay()`` function.

        **Description:**
            - This function queries for the additional delay time for the specified channels.

        **TSP Syntax:**

        ::

            - channel.getdelay()

        Args:
            channel_list: A string listing the channels to query for their delay times; slot1,
                slot2, or allslots allowed.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getdelay("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getdelay()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getdmm(self, channel_list: str, setting: str) -> str:
        """Run the ``channel.getdmm()`` function.

        **Description:**
            - This function returns the setting for a channel DMM attribute.

        **TSP Syntax:**

        ::

            - channel.getdmm()

        Args:
            channel_list: List of channels to get from the DMM.
            setting: The DMM function or the parameter values to return for the selected channels.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getdmm("{channel_list}", {setting}))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getdmm()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getlabel(self, channel_number: str) -> str:
        """Run the ``channel.getlabel()`` function.

        **Description:**
            - This function returns the label associated with the specified channel.

        **TSP Syntax:**

        ::

            - channel.getlabel()

        Args:
            channel_number: A string that lists the channel to query for the associated label.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.getlabel("{channel_number}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getlabel()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getstate(self, channel_list: Optional[str] = None) -> str:
        """Run the ``channel.getstate()`` function.

        **Description:**
            - This function returns the state indicators of the channels in the instrument.

        **TSP Syntax:**

        ::

            - channel.getstate()

        Args:
            channel_list (optional): String specifying the channels to query; use normal channel
                list syntax; if no channels are defined, all slots are returnedall channels are
                returned.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (f'"{channel_list}"' if channel_list is not None else None,)
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.getstate({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getstate()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def gettype(self, channel_list: str) -> str:
        """Run the ``channel.gettype()`` function.

        **Description:**
            - This function returns the type associated with a channel.

        **TSP Syntax:**

        ::

            - channel.gettype()

        Args:
            channel_list: String specifying the channels to query, using normal channelList syntax.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.gettype("{channel_list}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.gettype()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def open(self, channel_list: str) -> None:
        """Run the ``channel.open()`` function.

        **Description:**
            - This command opens the specified channels and channel pairs.

        **TSP Syntax:**

        ::

            - channel.open()

        Args:
            channel_list: String listing the channels to open; allslots, slot1, and slot2 available,
                which open all channels on all slots, all channels on slot 1, or all channels on
                slot 2allslots and slot1 are available; both open all channels.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.open("{channel_list}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.open()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setdelay(self, channel_list: str, delay: float) -> None:
        """Run the ``channel.setdelay()`` function.

        **Description:**
            - This function sets additional delay time for specified channels.

        **TSP Syntax:**

        ::

            - channel.setdelay()

        Args:
            channel_list: A string listing the channels for which to add delay time.
            delay: Delay time for the selected channels; minimum is 0 seconds.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setdelay("{channel_list}", {delay})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setdelay()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setdmm(self, channel_list: str, setting: str, value: str) -> None:
        """Run the ``channel.setdmm()`` function.

        **Description:**
            - This function configures the DMM for a channel or group of channels.

        **TSP Syntax:**

        ::

            - channel.setdmm()

        Args:
            channel_list: List of channels for which to set the DMM.
            setting: The DMM function or the parameter for the function to be applied to the
                channelList; refer to Details and the tables following the examples.
            value: The function or attribute value.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setdmm("{channel_list}", {setting}, {value})'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setdmm()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setlabel(self, channel_number: str, labelname: str) -> None:
        """Run the ``channel.setlabel()`` function.

        **Description:**
            - This function sets the label associated with a channel.

        **TSP Syntax:**

        ::

            - channel.setlabel()

        Args:
            channel_number: A string that lists the channel for which to set the label.
            labelname: A string that contains the label for the channel in channelNumber, up to 19
                characters.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.setlabel("{channel_number}", "{labelname}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setlabel()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""A collection of classes and other helpful objects to use when generating SCPI drivers.

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.
"""
import re
import string

from collections import defaultdict
from typing import Any, cast, Optional, Set, Tuple, TYPE_CHECKING, Union

from .generic_commands import BaseCmd, END_OF_STRING_DIGITS, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice

MAX_CHANNELS = 8
MAX_DIGITAL_BITS = 16
END_OF_STRING_NUMBER = re.compile(r"(\d+)$")


####################################################################################################
# Classes
####################################################################################################
class BaseSCPICmd(BaseCmd):  # pylint: disable=too-few-public-methods
    """A class to better type hint a member of a SCPI command tree."""

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._device: Optional["PIDevice"] = device


class SCPICmdRead(BaseSCPICmd):
    """A class defining a queryable SCPI command."""

    def query(self) -> str:
        """Return the result of the SCPI query.

        Sends the query ``.cmd_syntax + '?'``.

        This method will query the device using the ``device.query()`` method and return the result
        after stripping out leading and trailing whitespace.

        Returns:
            The query response from the device.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(self._cmd_syntax + "?")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = "No PIDevice object was provided, the .query() method cannot be used."
            raise NoDeviceProvidedError(msg) from error

    def verify(self, value: Union[float, str]) -> Tuple[bool, str]:
        """Verify the return value from the SCPI query matches the value.

        Sends the query ``.cmd_syntax + '?'``.

        This method will use the ``device.query_response()`` method to verify that the value
        returned by the device matches the expected value. An ``AssertionError`` will be raised by
        default if the values don't match.

        Args:
            value: The value to verify against the returned value from the device.

        Returns:
            A tuple containing a boolean indicating if the values match and a string with the actual
            return value from the device.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query_response(  # type: ignore[union-attr]
                self._cmd_syntax + "?", value
            )
        except AttributeError as error:
            msg = "No PIDevice object was provided, the .verify() method cannot be used."
            raise NoDeviceProvidedError(msg) from error


class SCPICmdReadWithArguments(BaseSCPICmd):
    """A class defining a SCPI command that can be queried with arguments."""

    _WRAP_ARG_WITH_QUOTES = False

    def query(self, argument: str) -> str:
        """Return the result of the SCPI query.

        Sends the query ``.cmd_syntax + '? argument'``.

        This method will query the device using the ``device.query()`` method and return the result
        after stripping out leading and trailing whitespace.

        Args:
            argument: The argument to send with the query.

        Returns:
            The query response from the device.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        argument = '"' + argument.strip('"') + '"' if self._WRAP_ARG_WITH_QUOTES else argument
        try:
            return self._device.query(  # type: ignore[union-attr]
                self._cmd_syntax + f"? {argument}".strip()
            )
        except AttributeError as error:
            msg = "No PIDevice object was provided, the .query() method cannot be used."
            raise NoDeviceProvidedError(msg) from error

    def verify(self, argument: str, value: Union[float, str]) -> Tuple[bool, str]:
        """Verify the return value from the SCPI query matches the value.

        Sends the query ``.cmd_syntax + '? argument'``.

        This method will use the ``device.query_response()`` method to verify that the value
        returned by the device matches the expected value. An ``AssertionError`` will be raised by
        default if the values don't match.

        Args:
            argument: The argument to send with the query.
            value: The value to verify against the returned value from the device.

        Returns:
            A tuple containing a boolean indicating if the values match and a string with the actual
            return value from the device.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        argument = '"' + argument.strip('"') + '"' if self._WRAP_ARG_WITH_QUOTES else argument
        try:
            return self._device.query_response(  # type: ignore[union-attr]
                self._cmd_syntax + f"? {argument}".strip(), value
            )
        except AttributeError as error:
            msg = "No PIDevice object was provided, the .verify() method cannot be used."
            raise NoDeviceProvidedError(msg) from error


class SCPICmdWrite(BaseSCPICmd):
    """A class defining a writable SCPI command."""

    _WRAP_ARG_WITH_QUOTES = False

    def write(self, value: Union[float, str], verify: bool = False) -> str:
        """Send the SCPI command (``.cmd_syntax``) with the provided value to the device.

        Args:
            value: The value to write to the device.
            verify: A boolean indicating that ``device.set_and_check()`` should be used.

        Returns:
            The value from the query when verification is enabled, otherwise an empty string.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        value = (
            '"' + value.strip('"') + '"'
            if self._WRAP_ARG_WITH_QUOTES and isinstance(value, str)
            else value
        )
        try:
            if verify or self._device.command_verification_enabled:  # type: ignore[union-attr]
                return self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax, value
                )
            self._device.write(f"{self._cmd_syntax} {value}")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = "No PIDevice object was provided, the .write() method cannot be used."
            raise NoDeviceProvidedError(msg) from error
        return ""


class SCPICmdWriteNoArguments(BaseSCPICmd):
    """A class defining a writable SCPI command which doesn't have any arguments."""

    def write(self) -> None:
        """Send the SCPI command (``.cmd_syntax``) to the device.

        This method will use the ``device.write()`` method to send the command.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(self._cmd_syntax)  # type: ignore[union-attr]
        except AttributeError as error:
            msg = "No PIDevice object was provided, the .write() method cannot be used."
            raise NoDeviceProvidedError(msg) from error


class ValidatedDigitalBit(BaseSCPICmd):  # pylint: disable=too-few-public-methods
    """A mixin class that is used to validate the digital bit is valid.

    This class checks the digital bit number at the end of the SCPI syntax (``.cmd_syntax``
    attribute) to determine if it is a valid digital bit number (within the range of 0 to the
    maximum number of digital bits on the device).
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)

        # Validate the bit number is correct
        bit_number_list = set(range(getattr(device, "num_dig_bits_in_ch", MAX_DIGITAL_BITS)))
        bit_num = int(
            END_OF_STRING_DIGITS.search(self._cmd_syntax).group(1)  # type: ignore[union-attr]
        )
        if bit_num not in bit_number_list:
            msg = f"{bit_num=} is not a valid bit number, valid bit numbers: {bit_number_list}"
            raise ValueError(msg)


class ValidatedChannel(BaseCmd):  # pylint: disable=too-few-public-methods
    """A mixin class that is used to validate the channel is valid.

    This class checks the channel number (or string) at the end of the syntax (``.cmd_syntax``
    attribute) to determine if it is a valid channel number (within the range of the available
    channels on the device).
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)

        # Validate the channel
        channel: Union[str, int]
        valid_channels: Set[Union[str, int]]
        if device is not None:
            valid_channel_strings = set(device.all_channel_names_list)
        else:
            valid_channel_numbers = set(range(1, MAX_CHANNELS + 1))
            valid_channel_strings = {f"CH{x}" for x in valid_channel_numbers}

        # Find the current channel
        uses_integer_channels = all(ch_name[-1].isdigit() for ch_name in valid_channel_strings)
        if uses_integer_channels:
            valid_channels: Set[Union[str, int]] = {
                int(number_match)
                for ch_name in valid_channel_strings
                for number_match in END_OF_STRING_NUMBER.findall(ch_name)
            }
            if (digit_match := END_OF_STRING_DIGITS.search(self._cmd_syntax)) is not None:
                channel = int(digit_match.group(1))
            else:
                msg = f"No channel number was detected in the command syntax '{self._cmd_syntax}'"
                raise ValueError(msg)
        else:
            valid_channels = cast(Set[Union[str, int]], valid_channel_strings)
            # Check if the channel "number" is actually a letter
            if (last_letter := self._cmd_syntax[-1]) in string.ascii_lowercase:
                channel = last_letter
            else:
                msg = f"No channel was detected in the command syntax '{self._cmd_syntax}'"
                raise ValueError(msg)

        # Check if the channel is valid
        if channel not in valid_channels:
            msg = f"{channel=} is not a valid channel, valid channels: {valid_channels}"
            raise ValueError(msg)


class DefaultDictDeviceCommunication(defaultdict):  # type: ignore
    """A custom default dictionary that can be used to send/receive commands to/from a device.

    The ``.query()`` method is used when ``__getitem__()`` is called and the result of the query is
    returned instead of a standard value.

    The ``.write()`` method is used when ``__setitem__()`` is called.
    """

    def __init__(
        self,
        cmd_syntax: str,
        query_syntax: str,
        write_syntax: Optional[str] = None,
        device: Optional["PIDevice"] = None,
        **kwargs: Any,
    ) -> None:
        """Create an instance of the class.

        Args:
            default_factory: The factory function used to create new values in the dictionary.
            cmd_syntax: The general syntax of the command, the substring that will need
              to be replaced with the key's value should be ``{key}``.
            query_syntax: The syntax to send to the device as a query, the substring that will need
              to be replaced with the key's value should be ``{key}``.
            write_syntax: The syntax to write to the device, the substring that will need
              to be replaced with the key's value should be ``{key}``.
            device: The device to send the query to.
            kwargs: The keyword arguments.
        """
        super().__init__(**kwargs)  # pyright: ignore [reportUnknownMemberType]
        self._cmd_syntax = cmd_syntax
        self._query_syntax = query_syntax
        self._write_syntax = write_syntax
        self._device = device

    def __setitem__(self, key: Any, value: Any) -> None:
        """Send a command to the device if possible.

        Args:
            key: The mapping item to use to modify the syntax.
            value: The value to write to the device.

        Raises:
            AttributeError: Indicates that the attribute is read-only.
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        if self._write_syntax is None:
            msg = f"``{self._cmd_syntax.format(key=key)}`` is a read-only attribute"
            raise AttributeError(msg)
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax.format(key=key), value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    self._write_syntax.format(key=key).strip() + " " + str(value)
                )
        except AttributeError as error:
            msg = "No Device object was provided, unable to write the command to the device."
            raise NoDeviceProvidedError(msg) from error

    def __getitem__(self, key: Any) -> str:
        """Query the device and return the result.

        Args:
            key: The mapping item to include in the query.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax.format(key=key)
            return self._device.query(  # type: ignore[union-attr]
                self._query_syntax.format(key=key)
            )
        except AttributeError as error:
            msg = "No Device object was provided, unable to send query."
            raise NoDeviceProvidedError(msg) from error



"""The lock commands module.

These commands are used in the following models:
MDO3

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - LOCk:ALL
    - LOCk:FPanel {LOCKed|UNLOCKed}
    - LOCk:FPanel?
    - LOCk:MOUse {LOCKed|UNLOCKed}
    - LOCk:MOUse?
    - LOCk:NONe
    - LOCk:TOUCHscreen {LOCKed|UNLOCKed}
    - LOCk:TOUCHscreen?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class LockTouchscreen(SCPICmdWrite, SCPICmdRead):
    """The ``LOCk:TOUCHscreen`` command.

    **Description:**
        - This command enables or disables the touchscreen.

    **Usage:**
        - Using the ``.query()`` method will send the ``LOCk:TOUCHscreen?`` query.
        - Using the ``.verify(value)`` method will send the ``LOCk:TOUCHscreen?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``LOCk:TOUCHscreen value`` command.

    **SCPI Syntax:**

    ::

        - LOCk:TOUCHscreen {LOCKed|UNLOCKed}
        - LOCk:TOUCHscreen?

    **Info:**
        - ``LOCKed`` disables the touchscreen.
        - ``UNLOCKed`` enables the touchscreen.
    """


class LockNone(SCPICmdWriteNoArguments):
    """The ``LOCk:NONe`` command.

    **Description:**
        - This command enables the front panel, mouse, and touchscreen.

    **Usage:**
        - Using the ``.write()`` method will send the ``LOCk:NONe`` command.

    **SCPI Syntax:**

    ::

        - LOCk:NONe
    """


class LockMouse(SCPICmdWrite, SCPICmdRead):
    """The ``LOCk:MOUse`` command.

    **Description:**
        - This command enables or disables the mouse.

    **Usage:**
        - Using the ``.query()`` method will send the ``LOCk:MOUse?`` query.
        - Using the ``.verify(value)`` method will send the ``LOCk:MOUse?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``LOCk:MOUse value`` command.

    **SCPI Syntax:**

    ::

        - LOCk:MOUse {LOCKed|UNLOCKed}
        - LOCk:MOUse?

    **Info:**
        - ``LOCKed`` disables the mouse.
        - ``UNLOCKed`` enables the mouse.
    """


class LockFpanel(SCPICmdWrite, SCPICmdRead):
    """The ``LOCk:FPanel`` command.

    **Description:**
        - This command enables or disables the front panel buttons and knobs.

    **Usage:**
        - Using the ``.query()`` method will send the ``LOCk:FPanel?`` query.
        - Using the ``.verify(value)`` method will send the ``LOCk:FPanel?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``LOCk:FPanel value`` command.

    **SCPI Syntax:**

    ::

        - LOCk:FPanel {LOCKed|UNLOCKed}
        - LOCk:FPanel?

    **Info:**
        - ``LOCKed`` disables the front panel.
        - ``UNLOCKed`` enables the front panel.
    """


class LockAll(SCPICmdWriteNoArguments):
    """The ``LOCk:ALL`` command.

    **Description:**
        - This command disables the front panel, mouse, and touchscreen.

    **Usage:**
        - Using the ``.write()`` method will send the ``LOCk:ALL`` command.

    **SCPI Syntax:**

    ::

        - LOCk:ALL
    """


class Lock(SCPICmdRead):
    """The ``LOCk`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``LOCk?`` query.
        - Using the ``.verify(value)`` method will send the ``LOCk?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.all``: The ``LOCk:ALL`` command.
        - ``.fpanel``: The ``LOCk:FPanel`` command.
        - ``.mouse``: The ``LOCk:MOUse`` command.
        - ``.none``: The ``LOCk:NONe`` command.
        - ``.touchscreen``: The ``LOCk:TOUCHscreen`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "LOCk") -> None:
        super().__init__(device, cmd_syntax)
        self._all = LockAll(device, f"{self._cmd_syntax}:ALL")
        self._fpanel = LockFpanel(device, f"{self._cmd_syntax}:FPanel")
        self._mouse = LockMouse(device, f"{self._cmd_syntax}:MOUse")
        self._none = LockNone(device, f"{self._cmd_syntax}:NONe")
        self._touchscreen = LockTouchscreen(device, f"{self._cmd_syntax}:TOUCHscreen")

    @property
    def all(self) -> LockAll:
        """Return the ``LOCk:ALL`` command.

        **Description:**
            - This command disables the front panel, mouse, and touchscreen.

        **Usage:**
            - Using the ``.write()`` method will send the ``LOCk:ALL`` command.

        **SCPI Syntax:**

        ::

            - LOCk:ALL
        """
        return self._all

    @property
    def fpanel(self) -> LockFpanel:
        """Return the ``LOCk:FPanel`` command.

        **Description:**
            - This command enables or disables the front panel buttons and knobs.

        **Usage:**
            - Using the ``.query()`` method will send the ``LOCk:FPanel?`` query.
            - Using the ``.verify(value)`` method will send the ``LOCk:FPanel?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``LOCk:FPanel value`` command.

        **SCPI Syntax:**

        ::

            - LOCk:FPanel {LOCKed|UNLOCKed}
            - LOCk:FPanel?

        **Info:**
            - ``LOCKed`` disables the front panel.
            - ``UNLOCKed`` enables the front panel.
        """
        return self._fpanel

    @property
    def mouse(self) -> LockMouse:
        """Return the ``LOCk:MOUse`` command.

        **Description:**
            - This command enables or disables the mouse.

        **Usage:**
            - Using the ``.query()`` method will send the ``LOCk:MOUse?`` query.
            - Using the ``.verify(value)`` method will send the ``LOCk:MOUse?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``LOCk:MOUse value`` command.

        **SCPI Syntax:**

        ::

            - LOCk:MOUse {LOCKed|UNLOCKed}
            - LOCk:MOUse?

        **Info:**
            - ``LOCKed`` disables the mouse.
            - ``UNLOCKed`` enables the mouse.
        """
        return self._mouse

    @property
    def none(self) -> LockNone:
        """Return the ``LOCk:NONe`` command.

        **Description:**
            - This command enables the front panel, mouse, and touchscreen.

        **Usage:**
            - Using the ``.write()`` method will send the ``LOCk:NONe`` command.

        **SCPI Syntax:**

        ::

            - LOCk:NONe
        """
        return self._none

    @property
    def touchscreen(self) -> LockTouchscreen:
        """Return the ``LOCk:TOUCHscreen`` command.

        **Description:**
            - This command enables or disables the touchscreen.

        **Usage:**
            - Using the ``.query()`` method will send the ``LOCk:TOUCHscreen?`` query.
            - Using the ``.verify(value)`` method will send the ``LOCk:TOUCHscreen?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``LOCk:TOUCHscreen value`` command.

        **SCPI Syntax:**

        ::

            - LOCk:TOUCHscreen {LOCKed|UNLOCKed}
            - LOCk:TOUCHscreen?

        **Info:**
            - ``LOCKed`` disables the touchscreen.
            - ``UNLOCKed`` enables the touchscreen.
        """
        return self._touchscreen



# pylint: disable=line-too-long
# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The tsplink commands module.

These commands are used in the following models: SMU2601B_Pulse

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.
"""
from typing import Dict, Optional, TYPE_CHECKING

from .._helpers import (
    BaseTSPCmd,
    DefaultDictPassKeyToFactory,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


# pylint: disable=too-few-public-methods
class TsplinkTriggerItem(ValidatedDynamicNumberCmd, BaseTSPCmd):
    """The ``tsplink.trigger[N]`` command tree.

    Constants:
        - ``.EVENT_ID``: Selects the event that causes a trigger to be asserted on the digital
          output line as the edge detected on the specified TSP-Link line.
    """

    EVENT_ID = "tsplink.trigger[N].EVENT_ID"
    """str: Selects the event that causes a trigger to be asserted on the digital output line as the edge detected on the specified TSP-Link line."""  # noqa: E501

    def __init__(self, device: Optional["TSPDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        # pylint: disable=invalid-name
        self.EVENT_ID = self.EVENT_ID.replace(
            "[N]", f"[{self._cmd_syntax.rsplit('[', maxsplit=1)[-1].split(']', maxsplit=1)[0]}]"
        )


class Tsplink(BaseTSPCmd):
    """The ``tsplink`` command tree.

    Constants:
        - ``.TRIG_BYPASS``: Allows direct control of the line as a digital I/O line.
        - ``.TRIG_EITHER``: Detects rising-edge or falling-edge triggers as input. Asserts a TTL-low
          pulse for output.
        - ``.TRIG_FALLING``: Detects falling-edge triggers as input. Asserts a TTL-low pulse for
          output.
        - ``.TRIG_RISING``: If the programmed state of the line is high, the tsplink.TRIG_RISING
          mode behaves similarly to tsplink.TRIG_RISINGA.
          If the programmed state of the line is low, the tsplink.TRIG_RISING mode behaves similarly to
          tsplink.TRIG_RISINGM.
          Use tsplink.TRIG_RISINGA if the line is in the high output state.
          Use tsplink.TRIG_RISINGM if the line is in the low output state.
        - ``.TRIG_RISINGA``: Detects rising-edge triggers as input. Asserts a TTL-low pulse for
          output.
        - ``.TRIG_RISINGM``: Edge detection as an input is not available. Generates a TTL-high pulse
          as an output trigger.
        - ``.TRIG_SYNCHRONOUS``: Detects the falling-edge input triggers and automatically latches
          and drives the trigger line low. Asserts a TTL-low pulse as an output trigger.
        - ``.TRIG_SYNCHRONOUSA``: Detects the falling-edge input triggers and automatically latches
          and drives the trigger line low.
        - ``.TRIG_SYNCHRONOUSM``: Detects rising-edge triggers as an input. Asserts a TTL-low pulse
          for output.

    Properties/methods:
        - ``.trigger``: The ``tsplink.trigger[N]`` command tree.
    """  # noqa: E501

    TRIG_BYPASS = "tsplink.TRIG_BYPASS"
    """str: Allows direct control of the line as a digital I/O line."""
    TRIG_EITHER = "tsplink.TRIG_EITHER"
    """str: Detects rising-edge or falling-edge triggers as input. Asserts a TTL-low pulse for output."""  # noqa: E501
    TRIG_FALLING = "tsplink.TRIG_FALLING"
    """str: Detects falling-edge triggers as input. Asserts a TTL-low pulse for output."""
    TRIG_RISING = "tsplink.TRIG_RISING"
    """str: If the programmed state of the line is high, the tsplink.TRIG_RISING mode behaves similarly to tsplink.TRIG_RISINGA.
If the programmed state of the line is low, the tsplink.TRIG_RISING mode behaves similarly to tsplink.TRIG_RISINGM.
Use tsplink.TRIG_RISINGA if the line is in the high output state.
Use tsplink.TRIG_RISINGM if the line is in the low output state."""  # noqa: E501
    TRIG_RISINGA = "tsplink.TRIG_RISINGA"
    """str: Detects rising-edge triggers as input. Asserts a TTL-low pulse for output."""
    TRIG_RISINGM = "tsplink.TRIG_RISINGM"
    """str: Edge detection as an input is not available. Generates a TTL-high pulse as an output trigger."""  # noqa: E501
    TRIG_SYNCHRONOUS = "tsplink.TRIG_SYNCHRONOUS"
    """str: Detects the falling-edge input triggers and automatically latches and drives the trigger line low. Asserts a TTL-low pulse as an output trigger."""  # noqa: E501
    TRIG_SYNCHRONOUSA = "tsplink.TRIG_SYNCHRONOUSA"
    """str: Detects the falling-edge input triggers and automatically latches and drives the trigger line low."""  # noqa: E501
    TRIG_SYNCHRONOUSM = "tsplink.TRIG_SYNCHRONOUSM"
    """str: Detects rising-edge triggers as an input. Asserts a TTL-low pulse for output."""

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "tsplink") -> None:
        super().__init__(device, cmd_syntax)
        self._trigger: Dict[int, TsplinkTriggerItem] = DefaultDictPassKeyToFactory(
            lambda x: TsplinkTriggerItem(device, f"{self._cmd_syntax}.trigger[{x}]")
        )

    @property
    def trigger(self) -> Dict[int, TsplinkTriggerItem]:
        """Return the ``tsplink.trigger[N]`` command tree.

        Constants:
            - ``.EVENT_ID``: Selects the event that causes a trigger to be asserted on the digital
              output line as the edge detected on the specified TSP-Link line.
        """
        return self._trigger



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The lan commands module.

These commands are used in the following models:
DAQ6510, DMM6500, DMM7510, SMU2450, SMU2460, SMU2461, SMU2470

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - lan.ipconfig()
    - lan.lxidomain
    - lan.macaddress
"""
from typing import Optional, TYPE_CHECKING, Union

from .._helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class Lan(BaseTSPCmd):
    """The ``lan`` command tree.

    Constants:
        - ``.MODE_AUTO``: The instrument automatically assigns LAN settings.
        - ``.MODE_MANUAL``: You specify the LAN settings.
        - ``.OFF``: An open and close on the DST port (5030) closes all open LAN connections.
        - ``.ON``: The DST port needs to be opened, receive the login + <system password>, then
          close the DST port to close all open LAN connections. By turning DST protection ON, you
          are protecting LAN connections from being inadvertently closed by your IT department doing
          a port scan across the corporate network.

    Properties/methods:
        - ``.ipconfig()``: The ``lan.ipconfig()`` function.
        - ``.lxidomain``: The ``lan.lxidomain`` attribute.
        - ``.macaddress``: The ``lan.macaddress`` attribute.
    """

    MODE_AUTO = "lan.MODE_AUTO"
    """str: The instrument automatically assigns LAN settings."""
    MODE_MANUAL = "lan.MODE_MANUAL"
    """str: You specify the LAN settings."""
    OFF = "lan.OFF"
    """str: An open and close on the DST port (5030) closes all open LAN connections."""
    ON = "lan.ON"
    """str: The DST port needs to be opened, receive the login + <system password>, then close the DST port to close all open LAN connections.  By turning DST protection ON, you are protecting LAN connections from being inadvertently closed by your IT department doing a port scan across the corporate network."""  # noqa: E501

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "lan") -> None:
        super().__init__(device, cmd_syntax)

    @property
    def lxidomain(self) -> str:
        """Access the ``lan.lxidomain`` attribute.

        **Description:**
            - This attribute contains the LXI domain.

        **Usage:**
            - Accessing this property will send the ``print(lan.lxidomain)`` query.
            - Setting this property to a value will send the ``lan.lxidomain = value`` command.

        **TSP Syntax:**

        ::

            - lan.lxidomain = value
            - print(lan.lxidomain)

        **Info:**
            - ``domain``, the LXI domain number (0 to 255).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".lxidomain"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.lxidomain)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.lxidomain`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @lxidomain.setter
    def lxidomain(self, value: Union[str, float]) -> None:
        """Access the ``lan.lxidomain`` attribute.

        **Description:**
            - This attribute contains the LXI domain.

        **Usage:**
            - Accessing this property will send the ``print(lan.lxidomain)`` query.
            - Setting this property to a value will send the ``lan.lxidomain = value`` command.

        **TSP Syntax:**

        ::

            - lan.lxidomain = value
            - print(lan.lxidomain)

        **Info:**
            - ``domain``, the LXI domain number (0 to 255).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".lxidomain", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.lxidomain = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.lxidomain`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def macaddress(self) -> str:
        """Access the ``lan.macaddress`` attribute.

        **Description:**
            - This attribute describes the LAN MAC address.

        **Usage:**
            - Accessing this property will send the ``print(lan.macaddress)`` query.

        **TSP Syntax:**

        ::

            - print(lan.macaddress)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".macaddress"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.macaddress)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.macaddress`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def ipconfig(
        self,
        method: Optional[str] = None,
        ip_v4_address: Optional[str] = None,
        subnet_mask: Optional[str] = None,
        gateway: Optional[str] = None,
    ) -> str:
        """Run the ``lan.ipconfig()`` function.

        **Description:**
            - This function specifies the LAN configuration for the instrument.

        **TSP Syntax:**

        ::

            - lan.ipconfig()

        Args:
            method (optional): The method for configuring LAN settings; it can be one of the
                following values.
            ip_v4_address (optional): LAN IP address; must be a string specifying the IP address in
                dotted decimal notation.
            subnet_mask (optional): The LAN subnet mask; must be a string in dotted decimal
                notation.
            gateway (optional): The LAN default gateway; must be a string in dotted decimal
                notation.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    method,
                    f'"{ip_v4_address}"' if ip_v4_address is not None else None,
                    f'"{subnet_mask}"' if subnet_mask is not None else None,
                    f'"{gateway}"' if gateway is not None else None,
                )
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.ipconfig({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.ipconfig()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error
